Package: hyreg2
Type: Package
Title: Estimate Latent Classes on a Mixture of Continuous and
        Dichotomous Data
Version: 1.1.2
Authors@R: c(person(given = "Svenja",
                      family = "Elkenkamp",
                      role = c("aut", "cre"),
                      email = "svenja.elkenkamp@uni-bielefeld.de"),
               person(given = "Kim",
                      family = "Rand",
                      role = "aut"),
               person(given = "John",
                      family = "Grosser",
                      role = "aut"),
               person("EuroQol",
                      role = "fnd"))
Description: The hybrid model likelihood as described by Ramos-Goñi et al. (2017) <doi:10.1097/MLR.0000000000000283> is implemented and
      and embedded in a latent class framework. The package is based on 'flexmix' and among others contains an M-step-driver as described by Leisch (2004) <doi:10.18637/jss.v011.i08>.
      Users can, for example, estimate latent classes for EQ-5D value sets and address preference heterogeneity. Both uncensored and censored data are supported. 
      Furthermore, heteroscedasticity can be taken into account. It is possible to control for different covariates on the continuous and dichotomous  data and start values can differ
      between the expected latent classes. 
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.1
Imports: flexmix, bbmle, ggplot2, methods, stats, utils
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
Config/testthat/edition: 3
Depends: R (>= 2.10)
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-03-17 11:53:06 UTC; selkenkamp
Author: Svenja Elkenkamp [aut, cre],
  Kim Rand [aut],
  John Grosser [aut],
  EuroQol [fnd]
Maintainer: Svenja Elkenkamp <svenja.elkenkamp@uni-bielefeld.de>
Repository: CRAN
Date/Publication: 2026-03-17 12:40:09 UTC
Built: R 4.6.0; ; 2026-04-28 05:05:58 UTC; unix

# Generated by roxygen2: do not edit by hand

export(FLXMRhyreg)
export(FLXMRhyreg_het)
export(get_stv)
export(give_class)
export(hyreg2)
export(hyreg2_het)
export(plot_hyreg2)
export(summary_hyreg2)
export(the)
importFrom(bbmle,mle2)
importFrom(bbmle,summary)
importFrom(flexmix,flexmix)
importFrom(ggplot2,ggplot)
importFrom(methods,new)
importFrom(stats,as.formula)
importFrom(stats,dnorm)
importFrom(stats,model.matrix)
importFrom(stats,pnorm)
importFrom(stats,setNames)
importFrom(utils,globalVariables)

## ----chunk-1, message=FALSE, warning=FALSE------------------------------------
# simulate dataframe called simulated_data_norm

library(hyreg2)
library(flexmix)


### CLASS 1 ###

# Set parameters
set.seed(42)
n_samples_tto1 <- 200
n_samples_dce1 <- 100
sigma1 <- 1.0
theta1 <- 5
beta1 <- c(0.5, -0.3, 0.8)  # true values for beta

# simulate matrix
x_tto1 <- matrix(rnorm(n_samples_tto1 * length(beta1)), ncol = length(beta1))
x_dce1 <- matrix(rnorm(n_samples_dce1 * length(beta1)), ncol = length(beta1))



#Xb (linear predictor following formula xb = x1*beta1 + x2*beta2 + x3*beta3)
Xb_tto1 <- x_tto1 %*% beta1
Xb_dce1 <- (x_dce1 %*% beta1) * theta1


# generate y
y_tto1 <- rnorm(n_samples_tto1, mean = Xb_tto1, sd = sigma1)  # continuous outcomes
logistic_tmp1 <- 0.5 + 0.5 * tanh(Xb_dce1 / 2)
y_dce1 <- rbinom(n_samples_dce1, size = 1, prob = logistic_tmp1)  # binary outcomes



### dataset ###
data_tto1 <- data.frame(
  type = rep("TTO", n_samples_tto1),
  y = y_tto1,
  x1 = x_tto1[, 1],
  x2 = x_tto1[, 2],
  x3 = x_tto1[, 3]
)

data_dce1 <- data.frame(
  type = rep("DCE_A", n_samples_dce1),
  y = y_dce1,
  x1 = x_dce1[, 1],
  x2 = x_dce1[, 2],
  x3 = x_dce1[, 3]
)

simulated_data1 <- rbind(data_tto1, data_dce1)


### CLASS 2 ###

# Set parameters
set.seed(37)
n_samples_tto2 <- 200
n_samples_dce2 <- 100
sigma2 <- 0.5
theta2 <- 2
beta2 <- c(1.4, 2.3, -0.2)   # true values for beta


# simulate matrix
x_tto2 <- matrix(rnorm(n_samples_tto2 * length(beta2)), ncol = length(beta2))
x_dce2 <- matrix(rnorm(n_samples_dce2 * length(beta2)), ncol = length(beta2))


#  Xb (linear predictor following formula xb = x1*beta1 + x2*beta2 + x3*beta3)
Xb_tto2 <- x_tto2 %*% beta2
Xb_dce2 <- (x_dce2 %*% beta2) * theta2


# generate y
y_tto2 <- rnorm(n_samples_tto2, mean = Xb_tto2, sd = sigma2)  # continuous outcomes
logistic_tmp2 <- 0.5 + 0.5 * tanh(Xb_dce2 / 2)
y_dce2 <- rbinom(n_samples_dce2, size = 1, prob = logistic_tmp2)  # binary outcomes



### dataset ###
data_tto2 <- data.frame(
  type = rep("TTO", n_samples_tto2),
  y = y_tto2,
  x1 = x_tto2[, 1],
  x2 = x_tto2[, 2],
  x3 = x_tto2[, 3]
)

data_dce2 <- data.frame(
  type = rep("DCE_A", n_samples_dce2),
  y = y_dce2,
  x1 = x_dce2[, 1],
  x2 = x_dce2[, 2],
  x3 = x_dce2[, 3]
)

simulated_data2 <- rbind(data_tto2, data_dce2)


### DATASET with two classes ###
simulated_data1$class <- 1
simulated_data2$class <- 2

# set id numerbs
simulated_data1$id <- c(1:100, 1:100, 1:100)
simulated_data2$id <- c(101:200, 101:200, 101:200)


simulated_data_norm<- rbind(simulated_data1,simulated_data2)


# clean environment
rm(list = setdiff(ls(), "simulated_data_norm"))


## ----chunk-2, message=FALSE, warning=FALSE------------------------------------

# visualize true classes from simulated_data_norm
p <- plot_hyreg2(data = simulated_data_norm,
           x = "id",
           y = "y",
           id_col = "id",
           class_df_model = simulated_data_norm[,c("id","class")],
           colors = c("#F8766D","#00BFC4"))
p <- p +  ggplot2::labs(title = "Figure 1")
p


## ----chunk-3, message=FALSE, warning=FALSE, results='hide'--------------------

set.seed(277)
formula <- y ~  -1 + x1 + x2 + x3
k <- 2
stv <- setNames(c(0.2,0,1,1,1),c("x1","x2","x3","sigma","theta"))
control <- list(iter.max = 1000, verbose = 4)

mod <- hyreg2(formula = formula,
                     data =  simulated_data_norm,
                     type =  simulated_data_norm$type,
                     stv = stv,
                     k = k,
                     type_cont = "TTO",
                     type_dich = "DCE_A",
                     opt_method = "L-BFGS-B",
                     control = control,
                     latent = "both"
)


## ----chunk-4, message=FALSE, warning=FALSE------------------------------------

summary_hyreg2(mod)


## ----chunk-5, message=FALSE, warning=FALSE------------------------------------

# if model was estimated without grouping variable
simulated_data_norm$observation <- rownames(simulated_data_norm)


p <- plot_hyreg2(data = simulated_data_norm,
           x = "id",
           y = "y",
           id_col = "observation",
           class_df_model = give_class(data = simulated_data_norm,
                                 model = mod,
                                 id_col = "observation"))
p <- p +  ggplot2::labs(title = "Figure 2")
p


## ----chunk-6, message=FALSE, warning=FALSE------------------------------------

# only continuous datapoints 
p <- plot_hyreg2(data = simulated_data_norm,
           x = "id",
           y = "y",
           id_col = "observation",
           class_df_model = give_class(data = simulated_data_norm,
                                 model = mod,
                                 id_col = "observation"),
           type_to_plot = list("type","TTO"))
p <- p +  ggplot2::labs(title = "Figure 3")
p



## ----chunk-7, message=FALSE, warning=FALSE------------------------------------

# only dichotomous datapoints
p <- plot_hyreg2(data = simulated_data_norm,
           x = "id",
           y = "y",
           id_col = "observation",
           class_df_model = give_class(data = simulated_data_norm,
                                 model = mod,
                                 id_col = "observation"),
           type_to_plot = list("type","DCE_A"))
p <- p +  ggplot2::labs(title = "Figure 4")
p



## ----chunk-8, message=FALSE, warning=FALSE------------------------------------

# ratio of correct classified datapoints
(sum(mod@cluster == simulated_data_norm$class))/dim(simulated_data_norm)[1]



# cross table of observation types and assigned classes
proof <- merge(simulated_data_norm,
                 give_class(simulated_data_norm,mod,"observation"), by = "observation")
table(proof$type,proof$mod_comp)





## ----chunk-9, message=FALSE, warning=FALSE, results='hide'--------------------
set.seed(227)
formula <- y ~  -1 + x1 + x2 + x3 | id
k <- 2
stv <- setNames(c(0.2,0,1,1,1),c("x1","x2","x3","sigma","theta"))
control <- list(iter.max = 1000, verbose = 4)

mod_id <- hyreg2(formula = formula,
                     data =  simulated_data_norm,
                     type =  simulated_data_norm$type,
                     stv = stv,
                     k = k,
                     type_cont = "TTO",
                     type_dich = "DCE_A",
                     opt_method = "L-BFGS-B",
                     control = control,
                     latent = "both"
)


## ----chunk-9a, message=FALSE, warning=FALSE-----------------------------------

summary_hyreg2(mod_id)



## ----chunk-10, message=FALSE, warning=FALSE-----------------------------------

p <- plot_hyreg2(data = simulated_data_norm,
           x ="id",
           y = "y",
           id_col ="id",
           class_df_model = unique(give_class(data = simulated_data_norm,
                                        model = mod_id,
                                        id_col = "id")))
p <- p +  ggplot2::labs(title = "Figure 5")
p



## ----chunk-13, message=FALSE, warning=FALSE-----------------------------------

# ratio of correct classified datapoints
(sum(mod_id@cluster == simulated_data_norm$class))/dim(simulated_data_norm)[1]


# cross table of observation types and assigned classes

proof <- merge(simulated_data_norm,
                 unique(give_class(simulated_data_norm,mod_id,"id")), by = "id")
table(proof$type,proof$mod_comp)




## ----chunk-14, message=FALSE, warning=FALSE, results='hide'-------------------
set.seed(227)
formula <- y ~  -1 + x1 + x2 + x3 | id
k <- 2
stv <- setNames(c(0.2,0,1,1,1),c("x1","x2","x3","sigma","theta"))
control <- list(iter.max = 1000, verbose = 4)

mod_cont <- hyreg2(formula = formula,
                     data =  simulated_data_norm,
                     type =  simulated_data_norm$type,
                     stv = stv,
                     k = k,
                     type_cont = "TTO",
                     type_dich = "DCE_A",
                     opt_method = "L-BFGS-B",
                     control = control,
                     latent = "cont",
                     id_col = "id"
)

## ----chunk-14a, message=FALSE, warning=FALSE----------------------------------

summary_hyreg2(mod_cont)


## ----chunk-15, message=FALSE, warning=FALSE-----------------------------------

p <- plot_hyreg2(data = simulated_data_norm,
           x ="id",
           y = "y",
           id_col ="id",
           class_df_model = give_class(data = simulated_data_norm,
                                model = mod_cont,
                                id_col = "id"))
p <- p +  ggplot2::labs(title = "Figure 6")
p



## ----chunk-16, message=FALSE, warning=FALSE-----------------------------------

# ratio of correct classified datapoints
proof <- merge(unique(simulated_data_norm[,c("id","class")]),mod_cont[["id_classes"]], by = "id")
sum((proof$class == proof$mod_comp)/dim(proof)[1])


# cross table of observation types and assigned classes

proof <- merge(simulated_data_norm,
                mod_cont[[k+1]], by = "id")
table(proof$type,proof$mod_comp)




## ----chunk-18, message=FALSE, warning=FALSE, results='hide'-------------------

# upper bound for simulated_data_norm$y at 3
# new variable y_cens

for(i in 1:length(simulated_data_norm$y)){
  if(simulated_data_norm$y[i] <= 3){
    simulated_data_norm$y_cens[i] <- simulated_data_norm$y[i]
  }else{
    simulated_data_norm$y_cens[i] <- 3
  }
}


set.seed(227)
formula <- y_cens ~  -1 + x1 + x2 + x3 | id
k <- 2
stv <- setNames(c(0.2,0,1,1,1),c("x1","x2","x3","sigma","theta"))
control <- list(iter.max = 1000, verbose = 4)

mod_cens <- hyreg2(formula = formula,
                     data =  simulated_data_norm,
                     type =  simulated_data_norm$type,
                     stv = stv,
                     k = k,
                     type_cont = "TTO",
                     type_dich = "DCE_A",
                     opt_method = "L-BFGS-B",
                     control = control,
                     upper = 3,
                     latent = "cont",
                     id_col = "id"
)


## ----chunk-19, message=FALSE, warning=FALSE, results='hide', fig.show='hide'----
summary_hyreg2(mod_cens)


plot_hyreg2(data = simulated_data_norm,
           x ="id",
           y = "y_cens",
           id_col = "id",
           class_df_model = give_class(data = simulated_data_norm,
                                 model = mod_cens,
                                 id_col = "id"))




## ----chunk-22, message=FALSE, warning=FALSE, results='hide'-------------------
set.seed(227)
formula <- y ~  -1 + x1 + x2 + x3 | id
k <- 2
stv1 <- setNames(c(0.2,0,1,1,1),c("x1","x2","x3","sigma","theta"))
stv2 <- setNames(c(1,2,0.5,1,1),c("x1","x2","x3","sigma","theta"))
control <- list(iter.max = 1000, verbose = 4)

mod_stvl<- hyreg2(formula = formula,
                     data =  simulated_data_norm,
                     type =  simulated_data_norm$type,
                     stv = list(stv1,stv2),
                     k = k,
                     type_cont = "TTO",
                     type_dich = "DCE_A",
                     opt_method = "L-BFGS-B",
                     control = control,
                     latent = "cont",
                     id_col = "id"
)

summary_hyreg2(mod_stvl)


## ----chunk-24, message=FALSE, warning=FALSE, results='hide'-------------------

set.seed(227)
formula <- y ~  -1 + x1 + x2 + x3 | id
formula_sigma <- y ~ x2 + x3
k <- 2
stv1 <- get_stv(mod_cont[[1]])[-4]  # -4 since "sigma" is on fourth place in return vector
stv2 <- get_stv(mod_cont[[2]])[-4]  # -4 since "sigma" is on fourth place in return vector
stv_sigma <- setNames(rep(0.1,3), c("x2","x3","(Intercept)"))
control <- list(iter.max = 1000, verbose = 4)


mod_het <- hyreg2_het(formula = formula,
                     formula_sigma = formula_sigma,
                     data =  simulated_data_norm,
                     type =  simulated_data_norm$type,
                     stv = list(stv1,stv2),
                     stv_sigma = stv_sigma,
                     k = k,
                     type_cont = "TTO",
                     type_dich = "DCE_A",
                     opt_method = "L-BFGS-B",
                     control = control,
                     latent = "cont",
                     id_col = "id"
)

## ----chunk-24a, message=FALSE, warning=FALSE----------------------------------
summary_hyreg2(mod_het)


## ----chunk-23, message=FALSE, warning=FALSE, results='hide'-------------------
set.seed(227)
formula <- y ~  -1 + x1 + x2 + x3 | id
k <- 2
stv <- setNames(c(0.2,0,1,1,1),c(colnames(simulated_data_norm)[3:5],c("sigma","theta")))
control <- list(iter.max = 1000, verbose = 4)

mod_partialc <- hyreg2(formula = formula,
                     data =  simulated_data_norm,
                     type =  simulated_data_norm$type,
                     stv = stv,
                     k = k,
                     type_cont = "TTO",
                     type_dich = "DCE_A",
                     opt_method = "L-BFGS-B",
                     control = control,
                     latent = "cont",
                     id_col = "id",
                     variables_both = "x1",
                     variables_cont = "x3",
                     variables_dich = "x2"
)

summary_hyreg2(mod_partialc)


## ----chunk-26, message=FALSE, warning=FALSE-----------------------------------

# simulate vector y_non and add this to simulated_data_norm

#library(hyreg2)

### CLASS 1 ###

# Set parameters (same as in simulation above)
set.seed(42)
n_samples_tto1 <- 200
n_samples_dce1 <- 100
sigma1 <- 1.0
theta1 <- 5
beta_non1 <- c(0.5, -0.3, 0.8)  # true values for beta

# simulate matrix
x_tto1 <- matrix(rnorm(n_samples_tto1 * length(beta_non1)), ncol = length(beta_non1))
x_dce1 <- matrix(rnorm(n_samples_dce1 * length(beta_non1)), ncol = length(beta_non1))


### NON-classic ###
# x1,x2 and x3 are columnnames of the dataset, beta1, beta2 and beta3 are the parameters to be estimated
formula1 <- y ~ (x1 * beta1 + x2 * beta2 ) * (x1 *beta1 + x3 * beta3)
x_tto_non1 <- data.frame(x1 = x_tto1[,1], x2 = x_tto1[,2], x3 = x_tto1[,3])
x_dce_non1 <- data.frame(x1 = x_dce1[,1], x2 = x_dce1[,2], x3 = x_dce1[,3])
beta_non1 <- setNames(beta_non1[1:3], c("beta1","beta2","beta3"))

Xb_tto_non1 <- hyreg2:::eval_formula_non(formula1, x_tto_non1, beta_non1)
Xb_dce_non1 <- (hyreg2:::eval_formula_non(formula1, x_dce_non1, beta_non1)) * theta1


# generate y_non
y_tto_non1 <- rnorm(n_samples_tto1, mean = Xb_tto_non1, sd = sigma1)  # continuous outcomes
logistic_tmp_non1 <- 0.5 + 0.5 * tanh(Xb_dce_non1 / 2)
y_dce_non1 <- rbinom(n_samples_dce1, size = 1, prob = logistic_tmp_non1)  # binary outcomes



### CLASS 2 ###

# Set parameters (same as in simulation above)
set.seed(37)
n_samples_tto2 <- 200
n_samples_dce2 <- 100
sigma2 <- 0.5
theta2 <- 2
beta_non2 <- c(1.4, 2.3, -0.2)   # true values for beta


# simulate matrix
x_tto2 <- matrix(rnorm(n_samples_tto2 * length(beta_non2)), ncol = length(beta_non2))
x_dce2 <- matrix(rnorm(n_samples_dce2 * length(beta_non2)), ncol = length(beta_non2))



### NON-classic ###
# x1,x2 and x3 are columnnames of the dataset, beta1, beta2 and beta3 are the parameters to be estimated
formula2 <- y  ~ (x1 * beta1 + x2 * beta2 ) * (x1 * beta1 + x3 * beta3)
x_tto_non2 <- data.frame(x1 = x_tto2[,1], x2 = x_tto2[,2], x3 = x_tto2[,3])
x_dce_non2 <- data.frame(x1 = x_dce2[,1], x2 = x_dce2[,2], x3 = x_dce2[,3])
beta_non2 <- setNames(beta_non2[1:3], c("beta1","beta2","beta3"))

Xb_tto_non2 <- hyreg2:::eval_formula_non(formula2, x_tto_non2, beta_non2)
Xb_dce_non2 <- (hyreg2:::eval_formula_non(formula2, x_dce_non2, beta_non2)) * theta2


# generate y_non
y_tto_non2 <- rnorm(n_samples_tto2, mean = Xb_tto_non2, sd = sigma2)  # continuous outcomes
logistic_tmp_non2 <- 0.5 + 0.5 * tanh(Xb_dce_non2 / 2)
y_dce_non2 <- rbinom(n_samples_dce2, size = 1, prob = logistic_tmp_non2)  # binary outcomes


simulated_data_norm$y_non <- c(y_tto_non1,y_dce_non1,y_tto_non2,y_dce_non2)

# clean environment
rm(n_samples_tto1,
   n_samples_dce1,
   sigma1,
   theta1,
   beta_non1,
   n_samples_tto2,
   n_samples_dce2,
   sigma2,
   theta2,
   beta_non2,
   logistic_tmp1,
   logistic_tmp2,
   logistic_tmp_non1,
   logistic_tmp_non2,
   x_tto_non1,
   x_tto_non2,
   x_dce_non1,
   x_dce_non2,
   Xb_tto_non1,
   Xb_tto_non2,
   Xb_dce_non1,
   Xb_dce_non2,
   x_dce1,
   x_dce2,
   x_tto1,
   x_tto2,
   y_dce_non1,
   y_dce_non2,
   y_tto_non1,
   y_tto_non2,
   formula1,
   formula2
)


## ----chunk-25, message=FALSE, warning=FALSE, results='hide'-------------------
set.seed(259)
formula <- y_non ~ (x1 * beta1 + x2 * beta2 ) * (x1 * beta1 + x3 * beta3)| id
k <- 2
stv <- setNames(c(0.2,0,1,1,1),c("beta1","beta2","beta3","sigma","theta"))
control <- list(iter.max = 1000, verbose = 4)

mod_non <- hyreg2(formula = formula,
                     data =  simulated_data_norm,
                     type =  simulated_data_norm$type,
                     stv = stv,
                     k = k,
                     type_cont = "TTO",
                     type_dich = "DCE_A",
                     opt_method = "L-BFGS-B",
                     control = control,
                     latent = "cont",
                     id_col = "id",
                     formula_type_classic = FALSE
)


## ----chunk-21, message=FALSE, warning=FALSE, results='hide'-------------------

summary_hyreg2(mod_non)

p <- plot_hyreg2(data = simulated_data_norm, 
           x ="id",
           y = "y_non",
           id_col = "id",
           class_df_model = give_class(data = simulated_data_norm,
                                 model = mod_non,
                                 id_col = "id"))
p <- p +  ggplot2::labs(title = "Figure 7")
p

# Helper functions not for user use #



######################
### refit function ###
######################

# internal function, not for direct user use
gendf <- function(list){
  df <- list()
  for(j in 1:length(list)){
    df[[j]] <- data.frame("Estimates" = c(list[[j]]@parameters[["coef"]], #  maybe include an other apply function here?
                                          list[[j]]@parameters[["sigma"]],
                                          list[[j]]@parameters[["theta"]]),
                          "Std Error" = bbmle::summary(list[[j]]@parameters[["fit_mle"]])@coef[,2],
                          "pvalue" =bbmle::summary(list[[j]]@parameters[["fit_mle"]])@coef[,4])
    # give AIC, loglik here as additional arguments?
  }

  return(df)
}


### refit function ###

# get parametervalues of models
# object is outcome of hyreg2
refit <- function(object){
  if(is.list(object)){
    out <- lapply(object, function(k){
      comp <- k@components
      out_list <- lapply(comp, function(j){gendf(j)})
    })
  }else{
    comp <- object@components
    out <- lapply(comp, function(j){gendf(j)})
  }
  return(out)
}






### HELPER FUNCTIONS FOR USE OF NON-CLASSIC FORMULAS ####

################################
### eval non-linear formulas ###
################################

# function to get Xb during M-step for non-classic case
eval_formula_non <- function(formula, data, stv, form_back = F){

  rhs <- formula[[3]]
  int <- is.element("INTERCEPT",all.vars(formula[[3]])) # intercept check

  # if(int & !is.element("(Intercept)", colnames(data))){
  #   data$"(Intercept)" <- rep(1,dim(data)[1])
  # }

  form_wght <- replace_vars(rhs, stv)

  # new env to be used in eval()
  env <- list2env(as.list(data), parent = parent.frame())

  # check for intercept
  if(int){
    intercept_val <- stv["INTERCEPT"]
    if(is.na(intercept_val)){
      intercept_val <- 0
    }
    form_wght <- call("+", intercept_val, form_wght)
  }

  if(form_back == TRUE){ # return formula before it is evaluated
    return(form_wght)
  }

  # get Xb, eval expression using stv (used in form_wght)
  xb <- eval(form_wght, envir = env)

  return(xb)
}




# replace variables by numbers given in stv
# fct used in eval_formula_non
replace_vars <- function(formula, stv){

  if(is.symbol(formula)){

    name <- as.character(formula)

    if(is.element(name, names(stv))){
      return(stv[[name]])
    }else{
      return(formula)
    }

  }else if(is.call(formula)){
    return(as.call(lapply(formula, replace_vars, stv = stv)))
  }else{
    return(formula)
  }
}




#####################################
### get formula for flexmix input ###
#####################################

# which variables from formula are part of data colnames?
# create formula to be uses in flexmix call
# returns a formula which can be used in flexmix

get_data_vars <- function(formula, data) {

  # split up formula
  #formula_parts <- strsplit(deparse(formula), "\\|")[[1]]

  # allow long formulas
  formula_str <- paste(deparse(formula, width.cutoff = 500), collapse = " ")
  formula_parts <- strsplit(formula_str, "\\|")[[1]]

  # find conditional part
  if(length(formula_parts) > 1){
    conditional <- trimws(formula_parts[2])
  }else{
    conditional <- NULL
  }

  # main formula
  main_formula <- as.formula(trimws(formula_parts[1]))

  # check which vars in formula are in data
  vars_data <- intersect(all.vars(main_formula[[3]]), names(data))

  rhs <- paste(vars_data, collapse = " + ")

  # intercept
  int <- is.element("INTERCEPT",all.vars(main_formula[[3]]))
  if(!int){
    rhs <- paste("-1 +", rhs)
  }

  # new main formula
  new_formula_str <- paste(deparse(main_formula[[2]]), "~", rhs)

  # add conditional part
  if (!is.null(conditional)) {
    new_formula_str <- paste(new_formula_str, "|", conditional)
  }


  return(as.formula(new_formula_str))
}




###################################################
### data and formula handling chehcks in hyreg2 ###
###################################################

# simplify formula for only cont or dich variables
simplify_formula <- function(formula){

  if(is.call(formula)){
    fun <- as.character(formula[[1]])
    args <- as.list(formula[-1])
    args <- lapply(args, simplify_formula)

    # Multiply
    if(fun == "*"){
      #if one argument is 0, everything is 0
      if (any(vapply(args, function(a) is.numeric(a) && a == 0, logical(1)))) {
        return(0)
      }

      # 1 * x → x
      args <- Filter(function(a) !(is.numeric(a) && a == 1), args)
      if(length(args) == 0){
        return(1)
      }
      if(length(args) == 1){
        return(args[[1]])
      }
      return(as.call(c(list(as.name("*")), args)))
    }

    # addition
    if (fun == "+") {

      # 0 + x → x
      args <- Filter(function(a) !(is.numeric(a) && a == 0), args)
      if(length(args) == 0){
        return(0)
      }
      if(length(args) == 1){
        return(args[[1]])
      }
      return(as.call(c(list(as.name("+")), args)))
    }

    # return everything else without canges
    return(as.call(c(list(as.name(fun)), args)))
  }else{
    return(formula)
  }
}


# check which vars from formula are undefined here (not part of variables for one type of data/ stv)
formulacheck_variables <- function(formula, data, stv) {
  if (is.symbol(formula)) {
    name <- as.character(formula)
    if (name %in% c(colnames(data), names(stv))) {
      return(formula)
    } else {
      return(0)
    }
  } else if (is.call(formula)) {
    fun <- formula[[1]]
    args <- as.list(formula[-1])
    new_args <- lapply(args, formulacheck_variables, data = data, stv = stv)
    out <- as.call(c(list(fun), new_args))
    return(simplify_formula(out))
  } else {
    return(formula)
  }
}

# check formula for variables in stv,
# fct used in stv check for non-classic fcts
get_stv_vars <- function(formula, stv) {
  return(intersect(all.vars(formula[[3]]), names(stv)))
}

### SETTING UP PACKAGE ###

#' creating environment for package internal objects
#' @export
#' @importFrom utils globalVariables


the <- new.env(parent = emptyenv())
the$test <- TRUE


if (getRversion() >= "2.15.1") {
  utils::globalVariables(c("counter"))
}

#' decode classes by the model
#'
#' @description This function can be used to decode the classified classes by the model generated using
#' `hyreg2` or `hyreg2_het`
#'
#' @param data a `dataframe`, which was used to estimate the `model`
#' @param model a flexmix `model`object estimated using [hyreg2()] or [hyreg2_het()]
#' @param id_col `character`-string, name of grouping variable, which must be a column of the provided `data`.
#'          the parameter must be specified, if the provided `model` was estimated under control for `groups`
#'
#'
#' @return `dataframe` of two columns, first column named as provided `id_col`  or `"observation"` if `id_col` was not given as
#'          an input. second column named `"mod_comp"` indicating the assigned class for this group or observation
#'
#'
#' @examples
#' # estimate a model using simulated_data_norm
#'
#' ### using grouping variable id ####
#'formula <- y ~  -1 + x1 + x2 + x3 | id
#'k <- 1
#'stv <- setNames(c(0.2,0.2,0.2,1,1),c(colnames(simulated_data_norm)[3:5],c("sigma","theta")))
#'control <- list(iter.max = 1000, verbose = 4)
#'
#'hyflex_mod <- hyreg2(formula = formula,
#'                     data =  simulated_data_norm,
#'                     type =  simulated_data_norm$type,
#'                     stv = stv,
#'                     k = k,
#'                     type_cont = "TTO",
#'                     type_dich = "DCE_A",
#'                     opt_method = "L-BFGS-B",
#'                     control = control,
#'                     latent = "both",
#'                     id_col = "id"
#')

#' # use of function give_class
#' give_class(data = simulated_data_norm,
#' model = hyflex_mod,
#' id_col = "id")
#'
#'
#'
#' @author Svenja Elkenkamp & John Grosser
#' @export
#'
#'



give_class <- function(data,
                    model,
                    id_col = NULL) # id must be provided, if model was estimated using a grouping variable
  {

    if(!is.list(model)){
      if(!is.null(id_col)){
        ids_comp <- data.frame(data[,id_col],model@cluster)
        colnames(ids_comp) <- c(id_col,"mod_comp")
      }else{
        data$mod_comp <- model@cluster
        ids_comp <- data.frame(rownames(data),data[,"mod_comp"])
        colnames(ids_comp) <- c("observation","mod_comp")
      }

    }else{
      ids_comp <- model[[length(model)]]
      colnames(ids_comp) <- c(id_col,"mod_comp")
    }

    return(ids_comp)
}





#' plot function for `hyreg2`
#'
#' @description Function to visualize model results by `hyreg2` or `hygre2_het`
#'
#'
#' @param data a `dataframe`, which was used to estimate the `model` using [hyreg2()] or [hyreg2_het()]
#' @param x `charachter` string, column of `data` to be plotted on x-axis
#' @param y `charachter` string, column of `data` to be plotted on y-axis
#' @param id_col `charachter` sting, grouping variable, same as was given in `model`.
#'            if model was estimated without grouping, see Details
#' @param class_df_model `dataframe` of two columns indicating which group belongs to which class,
#'                  first column named as input `id_col`, second column named `"mod_comp"`.
#'                  this input can be generated using the [give_class()] function, see Details.
#' @param type_to_plot `list` of two `charachter` elements. First: `columnname` of column containing indicator for `type` of `data`,
#'                       Second: `value` of column `type`, that should be used for the plot, see details of [hyreg2()] inputs `type` and `type_cont`,`type_dich`
#' @param colors `charachter` vector, colors to be used in `ggplot`, default `NULL` - than colors are choosen automatically
#'
#'
#'
#' @return `ggplot` object visualizing x against y by classes
#'
#' @details
#' `id_col` must be provided anyway, even if the model was estimated without grouping variable.
#' Since there might be no grouping varibale in the `data`, we recommend to create a new column called `"observation"`
#' in data using the `rownames`/`observationnumbers` as `charachter` values and use this column as
#' input for `id_col` in `plot_hyreg2`, additionally you can use `class_df_model` =  `give_class(data,model,"observation")`,
#' see example
#'
#' @examples
#' # estimate a model using simulated_data_rnorm
#'
#'formula <- y ~  -1 + x1 + x2 + x3 | id
#'k <- 2
#'stv <- setNames(c(0.2,0.2,0.2,1,1),c(colnames(simulated_data_norm)[3:5],c("sigma","theta")))
#'control <- list(iter.max = 1000, verbose = 4)
#'
#'hyflex_mod <- hyreg2(formula = formula,
#'                     data =  simulated_data_norm,
#'                     type =  simulated_data_norm$type,
#'                     stv = stv,
#'                     k = k,
#'                     type_cont = "TTO",
#'                     type_dich = "DCE_A",
#'                     opt_method = "L-BFGS-B",
#'                     control = control,
#'                     latent = "cont",
#'                     id_col = "id"
#')

#'# plotting the variables id against y
#'plot_hyreg2(data = simulated_data_norm,
#'           x = "id",
#'           y = "y",
#'           id_col = "id",
#'           class_df_model = give_class(data = simulated_data_norm,
#'                                 model = hyflex_mod,
#'                                 id = "id"))
#'

#'
#' @author Svenja Elkenkamp & John Grosser
#' @importFrom ggplot2 ggplot
#' @export
#'
#'


plot_hyreg2 <- function(data,
                       x,
                       y,
                       id_col,
                       class_df_model,  # you can use give_class() to generate class_df_model
                       type_to_plot = NULL, #list of two elements e.g. list("type","TTO")
                       colors = NULL # optional colour vector
){


  colnames(class_df_model) <- c(id_col,"mod_comp")
  data <- merge(data, class_df_model, by = id_col)

  if(!is.null(type_to_plot)){
    data <- data[(data[,type_to_plot[[1]]]) == type_to_plot[[2]],]
  }


#require("ggplot2")

 p <-  ggplot2::ggplot(mapping = ggplot2::aes(x = data[,x], y = data[,y], color = as.character(data$mod_comp)))
 p <- p + ggplot2::geom_point(size = 1, alpha = 0.7)
 p <- p +  ggplot2::labs( title = "classification",
          x = paste0(x),
          y = paste0(y),
          color = "Class")
 p <- p + ggplot2::geom_jitter(width = 0.1, height = 0, size = 1) #alpha = 0.2) # verschieben der Punkte leicht zur Seite
 p <- p + ggplot2::theme_minimal()

  # colours
  if (!is.null(colors)) {
    p <- p + ggplot2::scale_color_manual(values = colors)
  }

 return(p)

}

#' M-step driver to be used in flexmix
#'
#' @description Function used in flexmix M-Step to estimate hybrid model
#'
#' @param formula model `formula`, automatically provided by `hyreg2` and [flexmix::flexmix()]
#' @param family default `"hyreg"`, needed for [flexmix::flexmix()]
#' @param type `character` vector containing the indicator whether that datapoint (row)
#'  contains continuous or dichotomous data, see Details of [hyreg2]
#' @param type_cont value of `type` referring to continuous `data`, see Details of [hyreg2]
#' @param type_dich value of `type` referring to dichotomous `data`, see Details of [hyreg2]
#' @param variables_both `character` vector; variables to be fitted on both continuous and dichotomous data.
#'   see Details of [hyreg2]
#' @param variables_cont `character` vector; variables to be fitted only on continous data. see Details of [hyreg2]
#' @param variables_dich character vactor; variables to be fitted only on dichotomous data. see Details of [hyreg2]
#' @param stv `named vector` or `list` of named vectors containing start values for all coefficients from
#'  `formula`, including `theta`, see Details of [hyreg2::hyreg2]
#' @param offset offset as in [flexmix::flexmix()], default `NULL`
#' @param optimizer `character`, optimizer to be used in [bbmle::mle2()], default `"optim"`
#' @param opt_method `character`, optimization method to be used in `optimizer`, default `"BFGS"`
#' @param lower  lower bound for censored data. If this is used, `opt_method` must be
#'  set to `"L-BFGS-B"`, default `-INF`,
#' @param upper  upper bound for censored data. If this is used, `opt_method` must be
#'  set to `"L-BFGS-B"`,default `INF`
#' @param formula_type_classic `logical`; is the provided `formula` a typical R formula containing only variables
#'   or does it include variables and parameters? default `TRUE`
#' @param ... additional arguments for [flexmix::flexmix()] or [bbmle::mle2()]
#'
#' @return a `model` object, that can be used in [hyreg2] as input for parameter `model` in [flexmix::flexmix()]
#'
#'
#' @return a model object, that can be used in hyreg2 as input for parameter model in flexmix::flexmix
#'
#' @author Svenja Elkenkamp and Kim Rand
#'
#' @importFrom methods new
#' @importFrom stats as.formula dnorm model.matrix pnorm setNames
#'
#' @examples
#'
#'formula <- y ~  -1 + x1 + x2 + x3
#'the$k <- 2
#'stv <- setNames(c(0.2,0,1,1,1),c(colnames(simulated_data_norm)[3:5],c("sigma","theta")))
#'
#'
#'
#' x <- model.matrix(formula,simulated_data_norm)
#' y <- simulated_data_norm$y
#' w <- 1

#'model <- FLXMRhyreg(formula = formula,
#'                     family=c("hyreg"),
#'                     type =  simulated_data_norm$type,
#'                     stv = stv,
#'                     type_cont = "TTO",
#'                     type_dich = "DCE_A",
#'                     opt_method = "L-BFGS-B",
#'                     control = list(iter.max = 1000, verbose = 4),
#'                     offset = NULL,
#'                     optimizer = "optim",
#'                     variables_both =  names(stv)[!is.element(names(stv),c("sigma","theta"))],
#'                     variables_cont = NULL,
#'                     variables_dich = NULL,
#'                     lower = -Inf,
#'                     upper = Inf,
#')


#' @importFrom flexmix flexmix
#' @importFrom  bbmle mle2
#' @importFrom  bbmle summary
#' @export




### FLXMRhyreg ###


FLXMRhyreg <- function(formula = . ~ . ,
                       family = c("hyreg"),
                       type = NULL,
                       type_cont = NULL,
                       type_dich = NULL,
                       variables_both = NULL,
                       variables_cont = NULL,
                       variables_dich = NULL,
                       stv = NULL, # has to include starting values for sigma and theta as well
                       offset = NULL,
                       opt_method = "BFGS",
                       optimizer = "optim",
                       lower = -Inf,
                       upper = Inf,
                       formula_type_classic = TRUE,
                       ...
)
{
  family <- match.arg(family)

  # refit function has to depend on x,y,w.
  hyregrefit <- function(x, y, w) {
    warning(paste0("Not defined"))
    return(NA)
  }

  z <- new("FLXMRglm", weighted = TRUE, formula = formula,
           name = paste("FLXMRhyreg"), offset = offset,
           family = "hyreg", refit = hyregrefit)

  z@preproc.y <- function(x){
    if (ncol(x) > 1)
      stop(paste("for the", family, "family y must be univariate"))
    x
  }

  if(family=="hyreg"){
    z@defineComponent <- function(para) {  # we get para from the first estimation with start values

      predict <- function(x, type, ...) {
        dotarg = list(...)
        if("offset" %in% names(dotarg)) offset <- dotarg$offset


        if(formula_type_classic == TRUE){

          # CLASSIC
          if(type == type_cont){
            p <- x %*% para$coef[is.element(names(para$coef),c(variables_cont,variables_both))]  # Xb in xreg
          }
          if(type == type_dich){
            p <- (x %*% para$coef[is.element(names(para$coef),c(variables_dich,variables_both))]) * para$theta
          }
        }else{

          # NON-CLASSIC
          if(type == type_cont){
            p <- eval_formula_non(the$formula_non,
                                  the$data,
                                  para$coef[is.element(names(para$coef),
                                                       c(variables_cont,variables_both))])
          }
          if(type == type_dich){
            p <- eval_formula_non(the$formula_non,
                                  the$data,
                                  para$coef[is.element(names(para$coef),
                                                       c(variables_dich,variables_both))]) * para$theta
          }
        }
        if (!is.null(offset)) p <-  p + offset
        p
      }



      logLik <- function(x, y, sigma = para$sigma, theta = para$theta, return_vector = TRUE, ...){

        # prepare data
        # choose subset of x and y depending on type

        y1 <- y[type == type_cont]
        y2 <-  y[type == type_dich]


        sigma <- exp(sigma)

        if(isTRUE(formula_type_classic)){

          # classic
          x1 <- x[type == type_cont, c(variables_cont, variables_both)]
          x2 <- x[type == type_dich, c(variables_dich, variables_both)]

          if(length(c(variables_cont, variables_both)) == 1){
            Xb1 <- as.matrix(x1 * para$coef[c(variables_cont, variables_both)])
            colnames(Xb1) <- c(variables_cont, variables_both)
          }else{
            Xb1 <- x1 %*% para$coef[colnames(x1)]
          }
          if(length(c(variables_dich,variables_both)) == 1){
            Xb2 <- as.matrix( (x2 * para$coef[c(variables_dich, variables_both)]) * exp(theta) )
            colnames(Xb2) <- c(variables_dich, variables_both)
          }else{
            Xb2 <-(x2 %*% para$coef[colnames(x2)]) * exp(theta)
          }
        }else{

          # non-classic
          Xb1 <- as.matrix(eval_formula_non(the$formula_cont, # formula_non only for cont
                                            the$data[type == type_cont,],
                                            para$coef[c(variables_cont,variables_both)]))
          Xb2 <- as.matrix(eval_formula_non(the$formula_dich, # formula_non only for dich
                                            the$data[type == type_dich,],
                                            para$coef[c(variables_dich,variables_both)])) * exp(theta)  # only dich and both variables

        }

        # pvals and likelihood
        logistic_tmp <- .5+.5*tanh(Xb2/2)
        pvals2 <- log(y2 *logistic_tmp + (1-y2)* (1-logistic_tmp)) # dich_logistic

        pvals1 <- dnorm(y1, mean = Xb1, sd = sigma, log=TRUE) # cont_normal


        # for censored data
        if(upper != Inf ){
          censV <- y1 == upper
          pvals1[which(censV)] <- log(pnorm((Xb1[censV]-upper)/sigma, 0, 1) - pnorm((Xb1[censV]-Inf)/sigma, 0, 1))
        }

        if(lower != -Inf ){
          censV <- y1 == lower
          pvals1[which(censV)] <- log(pnorm((Xb1[censV]-(-Inf))/sigma, 0, 1) - pnorm((Xb1[censV]-lower)/sigma, 0, 1))
        }


        pvals <- c(pvals1,pvals2)

        pvals[pvals == -Inf] <- log(.Machine$double.xmin)
        pvals[pvals == Inf] <- log(.Machine$double.xmax)

        if(return_vector == TRUE){
          return(pvals)
        }else{
          return(-sum(pvals))         # get neg log L for optimizer
        }

      }

      new("FLXcomponent",
          parameters=list(coef = para$coef,
                          sigma = para$sigma,
                          theta = para$theta,
                          fit_mle = para$fit_mle),
          logLik = logLik,
          predict = predict,
          df = para$df)
    }


    z@fit <- function(x, y, w, component, ...){

      ### LIKELIHOOD FUNCTION to be used in ML Estimation ###

      # function to be used in mle, same as logLik but depending on stv and giving out the neg logL directly
      logLik2 <- function(stv){

        # prepare data
        y1 <- y[type == type_cont]
        y2 <-  y[type == type_dich]

        # prepare stv and sigma and theta
        sigma <- exp(stv[is.element(names(stv), c("sigma"))][[1]])
        theta <- exp(stv[is.element(names(stv), c("theta"))][[1]])
        stv_cont <- stv[!is.element(names(stv), c("sigma", "theta", variables_dich))]
        stv_dich <- stv[!is.element(names(stv), c("sigma","theta", variables_cont))]


        if(isTRUE(formula_type_classic)){

          # classic
          x1 <- x[type == type_cont, c(variables_cont, variables_both)]
          x2 <-  x[type == type_dich, c(variables_dich, variables_both)]

          if(length(stv_cont) == 1){
            Xb1 <- as.matrix(x1 * stv_cont)
            colnames(Xb1) <- stv_cont
          }else{
             Xb1 <- x1 %*% stv_cont[colnames(x1)]
          }
          if(length(stv_dich) == 1){
            Xb2 <- as.matrix(x2 * stv_dich)
            colnames(Xb2) <- stv_dich
          }else{
            Xb2 <- x2 %*% stv_dich[colnames(x2)]
          }
          Xb2 <- Xb2*theta
        }else{

          # non-classic
          Xb1 <- as.matrix(eval_formula_non(the$formula_cont, # formula_non only for cont
                                            the$data[type == type_cont,],
                                            stv_cont))
          Xb2 <- as.matrix(eval_formula_non(the$formula_dich, # formula_non only for dich
                                            the$data[type == type_dich,],
                                            stv_dich))

          Xb2 <- Xb2*theta
        }

        # pvals and likelihood
        logistic_tmp <- 0.5 + 0.5*tanh(Xb2/2)
        pvals2 <- log(y2 *logistic_tmp + (1-y2)* (1-logistic_tmp)) # dich_logistic

        pvals1 <- dnorm(y1, mean = Xb1, sd = sigma, log = TRUE) # cont_normal


        # for box constraints, censored data
        if(upper != Inf ){
          censV <- y1 == upper
          pvals1[which(censV)] <- log(pnorm((Xb1[censV]-upper)/sigma, 0, 1) - pnorm((Xb1[censV]-Inf)/sigma, 0, 1))
        }
        if(lower != -Inf ){
          censV <- y1 == lower
          pvals1[which(censV)] <- log(pnorm((Xb1[censV]-(-Inf))/sigma, 0, 1) - pnorm((Xb1[censV]-lower)/sigma, 0, 1))
        }

        pvals <- c(pvals1,pvals2)
        pvals[pvals == -Inf] <- log(.Machine$double.xmin)
        pvals[pvals == Inf] <- log(.Machine$double.xmax)

        # multiply weights for use in EM algo
        pvals_w <- pvals * w

        return(-sum(pvals_w))
      }

      # set names of inputs for logLik2
      if(isTRUE(formula_type_classic)){
        bbmle::parnames(logLik2) <- c(colnames(x), "sigma", "theta")
      }else{
        if(is.list(stv)){
          bbmle::parnames(logLik2) <- c(names(the$stv[[1]]))
        }else{
          bbmle::parnames(logLik2) <- c(names(the$stv))
        }
      }


      if(!exists("counter", envir = the)){

        the$counter <- 1

        if(is.list(stv)){
          stv_in <- stv[[the$counter]]
        }else{
          stv_in <- stv
        }

        fit_mle <- bbmle::mle2(minuslogl = logLik2,
                               start = stv_in,
                               optimizer = optimizer,
                               method = opt_method,
                               lower = -Inf,
                               upper = Inf)

      }else{
        if(the$counter < the$k){
          the$counter <- the$counter + 1

          if(is.list(stv)){
            stv_in <- stv[[the$counter]]
          }else{
            stv_in <- stv
          }

          fit_mle <- bbmle::mle2(minuslogl = logLik2,
                                 start = stv_in,
                                 optimizer = optimizer,
                                 method = opt_method,
                                 lower = -Inf,
                                 upper = Inf)



        }else{
          if(isTRUE(formula_type_classic)){
            stv_new <- setNames(c(component$coef, component$sigma, component$theta),
                                c(colnames(x), "sigma", "theta"))
          }else{
            if(is.list(stv)){
              stv_new <- setNames(c(component$coef, component$sigma, component$theta),
                                  c(names(the$stv[[1]]))) # order important, maybe flexiblize?
            }else{
              stv_new <- setNames(c(component$coef, component$sigma, component$theta),
                                  c(names(the$stv))) # order important, maybe flexiblize?
            }
          }

          fit_mle <- bbmle::mle2(minuslogl = logLik2,
                                 start = stv_new,
                                 optimizer = optimizer,
                                 method = opt_method,
                                 lower = -Inf,
                                 upper = Inf)
        }
      }


      z@defineComponent(para = list(coef = fit_mle@coef[!is.element(names(fit_mle@coef),
                                                                    c("sigma", "theta"))],
                                    df = ncol(x)+1, # not changed yet
                                    sigma = fit_mle@coef[is.element(names(fit_mle@coef), c("sigma"))],
                                    theta = fit_mle@coef[is.element(names(fit_mle@coef), c("theta"))],
                                    fit_mle = fit_mle,
                                    minLik = fit_mle@min)
      )
    }
  }

  z
}

#'  M-step driver to be used in flexmix accounting for heteroscedastisity
#'
#' @description Function used in flexmix M-Step to estimate hybrid model accounting for heteroscedastisity
#'
#' @param data a `data.frame` containing the data, see Details of [hyreg2_het]
#' @param formula linear model `formula`
#' @param family default `"hyreg"`, needed for [flexmix::flexmix()]
#' @param formula_sigma `formula` for estimation of `sigma` to account for heteroscedasticity, see Details [hyreg2_het]
#' @param type `character` vector containing the indicator whether that datapoint (row)
#'  contains continuous or dichotomous data, see Details of [hyreg2_het]
#' @param type_cont value of `type` referring to continuous `data`, see Details of [hyreg2_het]
#' @param type_dich value of `type` referring to dichotomous `data`, see Details of [hyreg2_het]
#' @param variables_both `character` vector; variables to be fitted on both continuous and dichotomous data.
#'   see Details of [hyreg2_het]
#' @param variables_cont `character` vector; variables to be fitted only on continous data. see Details of [hyreg2_het]
#' @param variables_dich character vactor; variables to be fitted only on dichotomous data. see Details of [hyreg2_het]
#' @param stv `named vector` or `list` of named vectors containing start values for all coefficients from
#'  `formula`, including `theta`, see Details of [hyreg2::hyreg2_het]
#' @param stv_sigma `named vector` with start values for sigma estimation.
#'  Have to be the same variables as given in `formula_sigma`, see Details of [hyreg2_het]
#' @param offset offset as in [flexmix::flexmix()], default `NULL`
#' @param optimizer `character`, optimizer to be used in [bbmle::mle2()], default `"optim"`
#' @param opt_method `character`, optimization method to be used in `optimizer`, default `"BFGS"`
#' @param lower  lower bound for censored data. If this is used, `opt_method` must be
#'  set to `"L-BFGS-B"`, default `-INF`,
#' @param upper  upper bound for censored data. If this is used, `opt_method` must be
#'  set to `"L-BFGS-B"`,default `INF`
#' @param ... additional arguments for [flexmix::flexmix()] or [bbmle::mle2()]
#'
#' @return a `model` object, that can be used in [hyreg2_het] as input for parameter `model` in [flexmix::flexmix()]
#'
#' @importFrom methods new
#' @importFrom stats as.formula dnorm model.matrix pnorm setNames
#'
#' @author Svenja Elkenkamp and Kim Rand
#' @examples
#'
#' formula <- y ~  -1 + x1 + x2 + x3
#' formula_sigma <- y ~ x1 + x2 + x3
# 'k <- 1
#' stv <- setNames(c(0.2,0,1,1),c(colnames(simulated_data_norm)[3:5],c("theta")))
#' stv_sigma <- setNames(c(0.2,0.2,0.1,1),c(colnames(simulated_data_norm)[3:5],c("(Intercept)")))
#'
# 'rm(counter)
#'
#' x <- model.matrix(formula,simulated_data_norm)
#' y <- simulated_data_norm$y
#' w <- 1

#'model <- FLXMRhyreg_het( data = simulated_data_norm,
#'                      formula = formula,
#'                      formula_sigma = formula_sigma,
#'                     family=c("hyreg"),
#'                     type =  simulated_data_norm$type,
#'                     stv = stv,
#'                     stv_sigma = stv_sigma,
#'                     type_cont = "TTO",
#'                     type_dich = "DCE_A",
#'                     opt_method = "L-BFGS-B",
#'                     control = list(iter.max = 1000, verbose = 4),
#'                     offset = NULL,
#'                     optimizer = "optim",
#'                     variables_both =  names(stv)[!is.element(names(stv),c("theta"))],
#'                     variables_cont = NULL,
#'                     variables_dich = NULL,
#'                     lower = -Inf,
#'                     upper = Inf,
#')

#' @importFrom flexmix flexmix
#' @export




### FLXMRhyreg_het ###


FLXMRhyreg_het <- function(data,
                           formula = . ~ .,
                           formula_sigma = formula_sigma,
                           family = c("hyreg"),
                           type = NULL,
                           type_cont = NULL,
                           type_dich = NULL,
                           variables_both = NULL,
                           variables_cont = NULL,
                           variables_dich = NULL,
                           stv = NULL, # has to include starting value for theta but not sigma
                           stv_sigma = NULL,
                           offset = NULL,
                           opt_method = "BFGS",
                           optimizer = "optim",
                           lower = -Inf,
                           upper = Inf,
                           ...
)
{
  # rm(counter)
  family <- match.arg(family)

  # refit function has to depend on x,y,w.

  hyregrefit <- function(x, y, w) {
    warning(paste0("Not defined"))
    return(NA)
  }

  z <- new("FLXMRglm", weighted = TRUE, formula = formula,
           name = paste("FLXMRhyreg"), offset = offset,
           family ="hyreg", refit = hyregrefit)

  z@preproc.y <- function(x){
    if (ncol(x) > 1)
      stop(paste("for the", family, "family y must be univariate"))
    x
  }

  if(family=="hyreg"){
    z@defineComponent <- function(para) {  # we get para from the first estimation with start values

      ### NEW ###
      predict <- function(x, type, ...) {
        dotarg = list(...)
        if("offset" %in% names(dotarg)) offset <- dotarg$offset

        # change needed for non-classic formulas
        if(type == type_cont){
          p <- x %*% para$coef[is.element(names(para$coef), c(variables_cont, variables_both))]  # Xb in xreg
        }
        if(type == type_dich){
          p <- (x %*% para$coef[is.element(names(para$coef), c(variables_dich, variables_both))]) * para$theta
        }

        if (!is.null(offset)) p <-  p + offset
        p
      }



      logLik <- function(x, y, sigma = para$sigma, theta = para$theta, return_vector = TRUE, ...){
        #para$sigma must be a vector

        #prepare sigma
        xsigma <-  model.matrix(formula_sigma,data[type == type_cont,])
        colnames(xsigma) <- paste0(colnames(xsigma), "_h")
        sigma <- exp(xsigma %*% sigma[colnames(xsigma)])


        # prepare data
        x1 <- x[type == type_cont, c(variables_cont, variables_both)]
        x2 <-  x[type == type_dich, c(variables_dich, variables_both)]
        y1 <- y[type == type_cont]
        y2 <-  y[type == type_dich]


        if(length(c(variables_cont,variables_both)) == 1){
          Xb1 <- as.matrix(x1 * para$coef[c(variables_cont, variables_both)])
          colnames(Xb1) <- c(variables_cont, variables_both)
        }else{
          Xb1 <- x1 %*% para$coef[colnames(x1)]
        }
        if(length(c(variables_dich,variables_both)) == 1){
          Xb2 <- as.matrix( (x2 * para$coef[c(variables_dich, variables_both)]) * exp(theta) )
          colnames(Xb2) <- c(variables_dich, variables_both)
        }else{
          Xb2 <-(x2 %*% para$coef[colnames(x2)]) * exp(theta)
        }


        # pvals and likelihood
        logistic_tmp <- .5+.5*tanh(Xb2/2)
        pvals2 <- log(y2 *logistic_tmp + (1-y2)* (1-logistic_tmp)) # dich_logistic

        pvals1 <- dnorm(y1, mean = Xb1, sd = sigma, log = TRUE) # cont_normal


        # for box constraints
        if(upper != Inf ){
          censV <- y1 == upper
          pvals1[which(censV)] <- log(pnorm((Xb1[censV]-upper)/sigma[censV], 0, 1) - pnorm((Xb1[censV]-Inf)/sigma, 0, 1))
        }

        if(lower != -Inf ){
          censV <- y1 == lower
          pvals1[which(censV)] <- log(pnorm((Xb1[censV]-(-Inf))/sigma[censV], 0, 1) - pnorm((Xb1[censV]-lower)/sigma, 0, 1))
        }


        pvals <- c(pvals1,pvals2)

        pvals[pvals == -Inf] <- log(.Machine$double.xmin)
        pvals[pvals == Inf] <- log(.Machine$double.xmax)

        if(return_vector == TRUE){
          return(pvals)
        }else{
          return(-sum(pvals))         # get neg log L for optimizer
        }

      }

      new("FLXcomponent",
          parameters=list(coef = para$coef,
                          sigma = para$sigma,
                          theta = para$theta,
                          fit_mle = para$fit_mle),
          logLik=logLik,
          predict = predict,
          df = para$df)
    }


    z@fit <- function(x, y, w, component, ...){


      # function to use in mle, same as logLik but depending on stv and giving out the neg logL directly
      logLik2 <- function(stv){

        # prepare sigma
        stv_sigma <- stv[is.element(names(stv), names(stv_sigma))]
        stv <-  stv[!is.element(names(stv), names(stv_sigma))]

        xsigma <-  model.matrix(formula_sigma, data[type == type_cont,])
        colnames(xsigma) <- paste0(colnames(xsigma), "_h")
        sigma <- exp(xsigma %*% stv_sigma[colnames(xsigma)])

        # prepare data
        x1 <- x[type == type_cont, c(variables_cont, variables_both)]
        x2 <-  x[type == type_dich, c(variables_dich, variables_both)]
        y1 <- y[type == type_cont]
        y2 <-  y[type == type_dich]


        #prepare stv
        theta <- exp(stv[is.element(names(stv),c("theta"))][[1]])
        stv_cont <- stv[!is.element(names(stv),c("sigma", "theta", variables_dich))]
        stv_dich <- stv[!is.element(names(stv),c("sigma", "theta", variables_cont))]



        if(length(stv_cont) == 1){
          Xb1 <- as.matrix(x1 * stv_cont)
          colnames(Xb1) <- stv_cont
        }else{
          Xb1 <- x1 %*% stv_cont[colnames(x1)]
        }

        if(length(stv_dich) == 1){
          Xb2 <- as.matrix(x2 * stv_dich)
          colnames(Xb2) <- stv_dich
        }else{
          Xb2 <- x2 %*% stv_dich[colnames(x2)]
        }

        Xb2 <- Xb2*theta


        # pvals and Likelihood calculation
        logistic_tmp <- 0.5 + 0.5*tanh(Xb2/2)
        pvals2 <- log(y2 *logistic_tmp + (1-y2)* (1-logistic_tmp)) # dich_logistic

        pvals1 <- dnorm(y1, mean = Xb1, sd = sigma, log = TRUE) # cont_normal

        # for box constraints:
        #in xreg:
        if(upper != Inf ){
          censV <- y1 == upper
          pvals1[which(censV)] <- log(pnorm((Xb1[censV]-upper)/sigma[censV], 0, 1) - pnorm((Xb1[censV]-Inf)/sigma, 0, 1))
        }

        if(lower != -Inf ){
          censV <- y1 == lower
          pvals1[which(censV)] <- log(pnorm((Xb1[censV]-(-Inf))/sigma[censV], 0, 1) - pnorm((Xb1[censV]-lower)/sigma, 0, 1))
        }


        pvals <- c(pvals1,pvals2)
        pvals[pvals == -Inf] <- log(.Machine$double.xmin)
        pvals[pvals == Inf] <- log(.Machine$double.xmax)


        # use weights for EM algo
        pvals_w <- pvals * w


        return(-sum(pvals_w))
      }


      # changes names of stv_sigma to be able to get estimates for it
      names(stv_sigma) <- paste0(names(stv_sigma), "_h")
      bbmle::parnames(logLik2) <- c(colnames(x), "theta", names(stv_sigma)) # set names of inputs for logLik2



      ### MODEL ESTIMATION ###
      if(!exists("counter", envir = the)){
        the$counter <- 1


        if(is.list(stv)){
          stv_in <- stv[[the$counter]]
        }else{
          stv_in <- stv # without sigma !
        }
        if(is.list(stv_sigma)){
          stv_sigma_in <- stv_sigma[[the$counter]]
        }else{
          stv_sigma_in <- stv_sigma
        }


        fit_mle <- bbmle::mle2(minuslogl = logLik2,
                               start = c(stv_in, stv_sigma_in),
                               optimizer = optimizer,
                               method = opt_method,
                               lower = lower,
                               upper = upper)

      }else{
        if(the$counter < the$k){
          the$counter <- the$counter + 1


          if(is.list(stv)){
            stv_in <- stv[[the$counter]]
          }else{
            stv_in <- stv # without sigma !
          }
          if(is.list(stv_sigma)){
            stv_sigma_in <- stv_sigma[[the$counter]]
          }else{
            stv_sigma_in <- stv_sigma
          }



          fit_mle <- bbmle::mle2(minuslogl = logLik2,
                                 start = c(stv_in, stv_sigma_in),
                                 optimizer = optimizer,
                                 method = opt_method,
                                 lower = lower,
                                 upper = upper)


        }else{
          stv_new <- setNames(c(component$coef, component$theta, component$sigma), c(colnames(x), "theta", names(stv_sigma)))
          fit_mle <- bbmle::mle2(minuslogl = logLik2,
                                 start = stv_new,
                                 optimizer = optimizer,
                                 method = opt_method,
                                 lower = lower,
                                 upper = upper)
        }
      }

      z@defineComponent(para = list(coef = fit_mle@coef[!is.element(names(fit_mle@coef), c(names(stv_sigma), "theta"))],
                                    df = ncol(x) + 1, #  how to change ???
                                    sigma = fit_mle@coef[is.element(names(fit_mle@coef), names(stv_sigma))],
                                    theta = fit_mle@coef[is.element(names(fit_mle@coef), c("theta"))],
                                    fit_mle = fit_mle,
                                    minLik = fit_mle@min)
      )
    }
  }

  z
}

#' Estimating hybrid models
#'
#' @description Estimation of hybrid model using continuous and dichotomous data e.g. EQ-5D data
#'
#' @param formula model `formula`, can be linear or non-linear. For non-linear formulas, variables and parameters
#' must be provided and `formula_type_classic` must be set to `FALSE`. Using `|xg` will include a grouping variable `xg`. see Details.
#' @param data a `data.frame` containing the data. see Details.
#' @param type either the name of the column in `data` containing an indicator of whether an
#'   observation is continuous or dichotomous (as `character`), or a `vector` containing the indicator.
#'   see Details.
#' @param type_cont value of `type` referring to continuous data. see Details.
#' @param type_dich Value of `type` referring to dichotomous data. see Details.
#' @param k `numeric`. Number of latent classes to be estimated via [flexmix::flexmix()].
#' @param control control list for [flexmix::flexmix()].
#' @param stv `named vector` or `list` of named vectors containing start values for all coefficients
#'  formula, including sigma and theta, see Details
#' @param offset offset as in [flexmix::flexmix()]
#' @param optimizer `charachter`,optimizer to be used in [bbmle::mle2()], default `"optim"`
#' @param opt_method `charachter`, optimization method to be used in optimizer, default `"BFGS"`
#' @param lower `numeric`, lower bound for censored data, default `-INF`. If this is used, `opt_method` must be set to `"L-BFGS-B"`,
#' @param upper `numeric`, upper bound for censored data, default `INF`. If this is used, `opt_method` must be set to `"L-BFGS-B"`,
#' @param latent `charachter`,data type to use in component identification, must be one of `"both"`, `"cont"` or `"dich"`,
#'  default  `“both”`,  see Details
#' @param id_col `character`, name of the grouping variable, only needed if `latent != "both”`, see Details
#' @param classes_only `logical`, default `FALSE`, indicates whether the function should perform only
#'  classification, rather than both classification and model estimation, only possible for `latent != "both"`,
#'   see Datails
#' @param variables_both `character vector`; variables to be fitted on both continuous and dichotomous data.
#'  If not specified, all variables from `formula` are used. If provided and not all variables from `formula`
#'  are included, `variables_cont` and `variables_dich` must be provided as well, while one of them can be `NULL`,
#'   see Details.
#' @param variables_cont `character` vector; variables to be fitted only on continous data. If provided,
#' `variables_both` and `variables_dich` must be provided as well.
#' @param variables_dich `character` vector; variables to be fitted only on dichotomous data, if provided,
#'  `variables_both` and `variables_cont` must be provided as well.
#' @param formula_type_classic `logical`; is the provided `formula` a classic R formula containing only variables (`TRUE`)
#'   or does it include both variables and parameters (`FALSE`)? default `TRUE`, see Details
#' @param ... additional arguments for [flexmix::flexmix()] or [bbmle::mle2()]
#'
#' @return model object of type `flexmix` or `list` of model objects of type `flexmix`.
#' Please note, that the estimates for `sigma` and `theta` are on a log-scale and have to be transformed using `exp()`to get the correct estimated values.
#'
#' @details
#' see details of different inputs listed below.
#'
#'
#'@section formula:
#' a classic R formula containing only variables(e.g.`y ~ x1 + x2 + …`) can be provided as well as a formula
#' including variables and parameters (non-classic) e.g. `y ~ x1 * beta1 + x2 * beta2`  or `y ~ 1/exp(x1 * beta1 + x2 * beta2)`,
#'  where `beta` are the parameters to be estimated and the`x`s are column names from the dataset.
#'  Non-linear models and the 8-parameter model for EQ-5D data can only be estimated using a non-classic formula.
#'  If the provided formula is non-classic, `formula_type_classic` must be set to `FALSE`.
#' When estimating an intercept, the formula must explicitly include a parameter named `"INTERCEPT"`(without a corresponding variable from the dataset)
#' Additionally, it is possible to include a grouping variable for repeated measures by using
#' `“| xg”` where `xg` is the column containing the group-memberships. The resulting formula will look
#' like this:  `y ~ x1 + x2 +… | xg`.  In `flexmix`, this is called the concomitant variable specification:
#' the  model is fit conditional on grouping, so that all observations with the same group are treated
#' as belonging together when computing likelihood contributions. One possible grouping variable can be
#' an id number to identify answers by the same participants. We highly recommend using a grouping variable,
#'  since otherwise the algorithm for k = 2 tends to classify all continuous data into one estimated class
#' and all dichotomous data into the other.
#'
#'
#' @section data:
#'  a dataframe having the following columns: all independent variables (x)
#'  and the dependent variable y used in `formula`, one column for the grouping variable xg if grouping
#'  should be used, e.g. id numbers of participants with repeated measurements, one column indicating
#'  if the observations belongs to continuous or dichotomous data with the entries `type_cont`
#'  and `type_dich` (e.g., for a column called `"type"` with the entries "TTO" for continuous datapoints
#'  and "DCE" for dichotomous datapoints, `type_cont` will be "TTO" and `type_dich` will be "DCE").
#'  One row should match one observation (one datapoint).
#'
#'
#' @section start values (stv):
#' if the same start values `stv` are to be used for all latent classes,
#' the given start values must be a `named vector`. Otherwise (if different start values are assumed for
#'  each latent class), a `list` of named vectors should be used . In this case, there must be one entry
#'  in the list for each latent class.  Each start value vector must include start values for sigma and
#'  theta. Currently, it is necessary to use the names `"sigma"` and `"theta"` for these values.
#'  If users are unsure for which variables start values must be provided (in the linear formula case), this can be checked by
#'  calling `colnames(model.matrix(formula,data))`. In this call, the `formula` should not include the
#'  grouping variable.
#'
#'
#' @section latent, id_col, classes_only:
#' in some situations, it can be useful to identify the latent classes on
#' only one `type` of data while estimating the model parameters on both `types` of data. In such cases,
#' the input variable `latent` can be used to specify on which type of data the classification should be done.
#'  If `“cont”` or `“dich”` is used, `formula` must contain a grouping variable and additionally the
#'  input parameter `id_col` must be specified and gives the name,
#'  i.e. a `character string`, of the grouping variable for classification. Some groups may be removed from
#'  the data, since they have only continuous or only dichotomous observations. Then in a first step,
#'  a model is estimated only on the continuous/dichotomous data and the achieved classification is stored.
#'  In a next step, model parameters are estimated separately for each identified class on both `types` of data
#'  using this classification. The output object of `hyreg2` in this case is a `list` of k models.
#'  Additionally, at position k+1 of the list, a data frame containing the corresponding classifications
#'  from the first step is returned. Each element k in the list contains the estimated parameters for one
#'  of the latent classes. When setting the input variable `classes_only` to `TRUE`, the second step is left
#'  out and the estimated classes from step one are given as output.
#'
#'
#' @section variables_both, variables_cont, variables_dich:
#' It is possible to specify partial coefficients,
#'  which are used only on continuous or dichotomous data.
#'*  Example:  Suppose different models should be specified for continuous and dichotomous  data:
#'*  Model continuous data: `y ~  x1 + x3`
#'*  Model dichotomous data: `y ~  x1 + x2`
#'*  The `formula` input to `hyreg2` must then include all parameters that occur in either model:
#'   `y ~ x1 + x2 + x3`
#'* The assignment of parameters to data types is then achieved via the input arguments `variables_both`,
#'   `variables_cont`, and `variables_dich`:
#'*  `variables_both` = `“x1”`,
#'*  `variables_cont` = `“x3”` and
#'* `variables_dich` = `“x2”`.
#'* Every variable included in the provided `formula` (except the grouping variable ) must appear in exactly
#'   one of these vectors. One of the `variables_` vectors can also be `NULL`, if no variables should be used only on this type of the data.

#' @author Svenja Elkenkamp, Kim Rand and John Grosser
#' @examples
#'
#'formula <- y ~  -1 + x1 + x2 + x3 | id
#'
#'k <- 2

#'stv <- setNames(c(0.2,0,1,1,1),c(colnames(simulated_data_norm)[3:5],c("sigma","theta")))
#'control = list(iter.max = 1000, verbose = 4)

#'rm(counter)

#'mod <- hyreg2(formula = formula,
#'                     data =  simulated_data_norm,
#'                     type =  simulated_data_norm$type, # also "type" would work
#'                     stv = stv,
#'                     k = k,
#'                     type_cont = "TTO",
#'                     type_dich = "DCE_A",
#'                     opt_method = "L-BFGS-B",
#'                     control = control,
#'                     latent = "cont",
#'                     id_col = "id"
#')
#'summary_hyreg2(mod)

#' @importFrom flexmix flexmix
#' @importFrom bbmle mle2
#' @export


hyreg2 <-function(formula,
                  data,
                  type,
                  type_cont,
                  type_dich,
                  k = 1,
                  control = NULL,
                  stv = NULL, # has to include starting values for sigma and theta as well
                  offset = NULL,
                  opt_method = "BFGS",
                  optimizer = "optim",
                  lower = -Inf,
                  upper = Inf,
                  latent = "both", # one of "both", "cont", "dich"
                  id_col = NULL, # as character
                  classes_only = FALSE,
                  variables_both = NULL,
                  variables_dich = NULL,
                  variables_cont = NULL,
                  formula_type_classic = TRUE,
                  # additional arguments for flexmix or optimizer ?
                  ...){

  dotarg <- list(...)

  if(is.character(type) & length(type) == 1 ){
    type <- data[,type]
  }

  # assign k to k in package environment to be used during M-step driver
  assign("k", k, envir = the)
  if(exists("counter", envir = the)){
    rm("counter", envir = the)
  }

  # prepare formula handling
  formula_string <- paste(deparse(formula), collapse = "")
  formula_parts <- strsplit(formula_string, "\\|")[[1]]
  formula_short <- as.formula(formula_parts[1])




  # STV CHECK BASED ON FORMULA_TYPE_CLASSIC
  if(isTRUE(formula_type_classic)){

    ### STV Check Classic ###
    # check stv for names in x (model.matrix(formula,data))

    # no stv
    if(!is.list(stv)){
      if(is.null(stv)){
        warning(paste0("Argument stv not provided. Setting all start values from formula to 0.1 and start values for sigma and theta to 1."))
        stv <- setNames(c(rep(0.1,dim(model.matrix(formula_short,data))[2]), 1, 1),
                        c(colnames(model.matrix(formula_short, data)), c("sigma", "theta")))
      }else{

        # one or more stv missing
        if(any(!is.element(colnames(model.matrix(formula_short,data)),
                           names(stv)))){
          miss <- colnames(model.matrix(formula_short,data))[!is.element(colnames(model.matrix(formula_short, data)),
                                                                         names(stv))]
          stop(paste0("Start value(s) missing for ", paste(miss, collapse = ", ") ,". Please provide start values for all relevant formula variables."
          ))
        }

        # theta missing
        if(!is.element("theta",
                       names(stv))){
          stv <- c(stv, setNames(1, "theta"))
          warning(paste0("Start value missing for theta, setting to 1."))
        }

        # sigma missing
        if(!is.element("sigma",
                       names(stv))){
          stv <- c(stv, setNames(1, "sigma"))
          warning(paste0("Start value missing for sigma, setting to 1."))
        }

        # stv for variables not in formula given, d.h. zu viele angegeben
        if(any(!is.element(names(stv),
                           c(colnames(model.matrix(formula_short, data)), "theta", "sigma")))){
          much <- names(stv)[!is.element(names(stv),
                                         c(colnames(model.matrix(formula_short, data)), "theta", "sigma"))]
          stop(paste0("Start values provided for variables not in formula: ",paste(much, collapse = ", ")))
        }
        #  check order in FLXMRhyreg
      }

    }else{ # stv is a list

      # warning(paste0("stv is a list. Please ensure that variables_both, variables_dich and variables_cont are provided"))

      for(i in 1:length(stv)){
        # one or more stv missing
        if(any(!is.element(colnames(model.matrix(formula_short, data)),
                           names(stv[[i]])))){
          miss <- colnames(model.matrix(formula_short, data))[!is.element(colnames(model.matrix(formula_short, data)),
                                                                          names(stv[[i]]))]
          stop(paste0("Start value(s) missing for ", paste(miss, collapse = ", ") ,". Please provide start values for all relevant formula variables."
          ))
        }

        # theta missing
        if(!is.element("theta",
                       names(stv[[i]]))){
          stv[[i]] <- c(stv[[i]], setNames(1, "theta"))
          warning(paste0("Start value missing for theta, setting to 1."))
        }

        # sigma missing
        if(!is.element("sigma",
                       names(stv[[i]]))){
          stv[[i]] <- c(stv[[i]], setNames(1,"sigma"))
          warning(paste0("Start value missing for sigma, setting to 1."))
        }

        # stv for variables not in formula given, d.h. zu viele angegeben
        if(any(!is.element(names(stv[[i]]),
                           c(colnames(model.matrix(formula_short, data)), "theta", "sigma")))){
          much <- names(stv[[i]])[!is.element(names(stv[[i]]),
                                              c(colnames(model.matrix(formula_short, data)), "theta", "sigma"))]
          stop(paste0("Start values provided for variables not in formula: ",paste(much, collapse = ", ")))
        }
      }
    }
  }else{  # close formula_type_classic TRUE

    # ADAPTIONS FOR NON-CLASSIC FORMULAS

    # save data and formula in the environment
    assign("formula_non", formula_short, envir = the)
    assign("data", data, envir = the)

    # transform formulas for flexmix inputs
    formula <- get_data_vars(formula, data)
    formula_short <- get_data_vars(formula_short, data)


    ### STV Check NON-CLASSIC ###

    # check stv for names in formula
    vars <- all.vars(the$formula_non[[3]])[!is.element(all.vars(the$formula_non[[3]]),
                                                       names(data))]

    # no stv
    if(!is.list(stv)){
      if(is.null(stv)){
        warning(paste0("Argument stv not provided. Setting all start values from formula to 0.1 and start values for sigma and theta to 1."))
        stv <- setNames(c(rep(0.1, length(vars)), 1, 1),
                        c(vars, c("sigma", "theta")))
      }else{

        # one or more stv missing
        if(any(!is.element(vars, names(stv)))){
          miss <- vars[!is.element(vars, names(stv))]
          stop(paste0("Start value(s) missing for ", paste(miss, collapse = ", ") ,". Please provide start values for all relevant formula variables."
          ))
        }

        # theta missing
        if(!is.element("theta",
                       names(stv))){
          stv <- c(stv, setNames(1,"theta"))
          warning(paste0("Start value missing for theta, setting to 1."))
        }

        # sigma missing
        if(!is.element("sigma",
                       names(stv))){
          stv <- c(stv, setNames(1,"sigma"))
          warning(paste0("Start value missing for sigma, setting to 1."))
        }

        # stv for variables not in formula given, d.h. zu viele angegeben
        if(any(!is.element(names(stv),
                           c(vars, "theta", "sigma")))){
          much <- names(stv)[!is.element(names(stv),
                                         c(vars, "theta", "sigma"))]
          stop(paste0("Start values provided for variables not in formula: ",paste(much, collapse = ", ")))
        }
        #  check order in FLXMRhyreg
      }

    }else{ # stv is a list

      # warning(paste0("stv is a list. Please ensure that variables_both, variables_dich and variables_cont are provided"))

      for(i in 1:length(stv)){
        # one or more stv missing
        if(any(!is.element(vars,
                           names(stv[[i]])))){
          miss <- vars[!is.element(vars,
                                   names(stv[[i]]))]
          stop(paste0("Start value(s) missing for ", paste(miss, collapse = ", ") ,". Please provide start values for all relevant formula variables."
          ))
        }

        # theta missing
        if(!is.element("theta",
                       names(stv[[i]]))){
          stv[[i]] <- c(stv[[i]], setNames(1,"theta"))
          warning(paste0("Start value missing for theta, setting to 1."))
        }

        # sigma missing
        if(!is.element("sigma",
                       names(stv[[i]]))){
          stv[[i]] <- c(stv[[i]], setNames(1, "sigma"))
          warning(paste0("Start value missing for sigma, setting to 1."))
        }

        # stv for variables not in formula given, d.h. zu viele angegeben
        if(any(!is.element(names(stv[[i]]),
                           c(vars, "theta", "sigma")))){
          much <- names(stv[[i]])[!is.element(names(stv[[i]]),
                                              c(vars, "theta", "sigma"))]
          stop(paste0("Start values provided for variables not in formula: ",paste(much, collapse = ", ")))
        }
      }
    }
  } # close formula_type_classic FALSE check (non-classic formulas)



  ### TYPE Check ###
  if(is.null(type) | is.null(type_dich) | is.null(type_cont)){
    stop(paste0("Argument(s) type, type_dich and/or typ_cont not provided."))
  }else{
    if(!is.element(type_dich, unique(type))){
      warning(paste0("Invalid type_dich: provided name not present in values of type"))
    }
    if(!is.element(type_cont, unique(type))){
      warning(paste0("Invalid type_cont: provided name not present in values of type"))
    }
  }



  ### VARIABALES Check ###
  if(!is.list(stv)){
    if(is.null(variables_both) & is.null(variables_dich) & is.null(variables_cont)){
      variables_both <- names(stv)[!is.element(names(stv),
                                               c("sigma", "theta"))]
    }else{
      if(any(!is.element(names(stv)[!is.element(names(stv),
                                                c("sigma", "theta"))],
                         c(variables_both, variables_dich, variables_cont)))){
        # check if all variables are included
        stop(paste0("All variables named in stv must be contained in exactly one of: (1) variables_both, (2) variables_dich or (3) variables_cont."))
        # alternative: do not provide any of the vectors
        # than all relevant variables are set to variables_both automatically
      }
      if(any(table(c(variables_both,variables_cont,variables_dich))>1)){
        stop(paste0("All variables named in stv must be contained in exactly one of: (1) variables_both, (2) variables_dich or (3) variables_cont."))
      }
    }

  }else{

    # if stv is a list, both classes have to depend on the same set of variables,
    # stv of different classes have same names (hence using stv[[1]] is okay here)
    # different set of variables for each class not supported yet!

    if(is.null(variables_both) & is.null(variables_dich) & is.null(variables_cont)){
      variables_both <- names(stv[[1]])[!is.element(names(stv[[1]]),
                                                    c("sigma", "theta"))]
    }else{
      if(any(!is.element(names(stv[[1]])[!is.element(names(stv[[1]]),
                                                     c("sigma", "theta"))],
                         c(variables_both,variables_dich,variables_cont)))){
        # check if all variables are included
        stop(paste0("All variables named in stv must be contained in exactly one of: (1) variables_both, (2) variables_dich or (3) variables_cont."))
        # alternative: do not provide any of the vectors
        # than all relevant variables are set to variables_both automatically
      }
      if(any(table(c(variables_both, variables_cont, variables_dich)) > 1)){
        stop(paste0("All variables named in stv must be contained in exactly one of: (1) variables_both, (2) variables_dich or (3) variables_cont."))
      }
    }
  }


  ### PREPARE M-STEP FOR NON-CLASSIC FORMULAS ###

  if(isFALSE(formula_type_classic)){

    # hard coding for stv as list because formulas must be the same for all components
    # change this if formulas can differ between components

    assign("stv", stv, envir = the)

    if(!is.list(stv)){
      assign("stv_cont", stv[!is.element(names(stv), c("sigma", "theta", variables_dich))], envir = the)
      assign("stv_dich", stv[!is.element(names(stv), c("sigma", "theta", variables_cont))], envir = the)
    }else{
      assign("stv_cont", stv[[1]][!is.element(names(stv[[1]]), c("sigma", "theta", variables_dich))], envir = the)
      assign("stv_dich", stv[[1]][!is.element(names(stv[[1]]), c("sigma", "theta", variables_cont))], envir = the)
    }

    assign("formula_cont", formulacheck_variables(the$formula_non, the$data, the$stv_cont), envir = the)
    assign("formula_dich", formulacheck_variables(the$formula_non, the$data, the$stv_dich), envir = the)
  }


  ### ESTIMATION ###
  if(latent == "both"){

    model <- list(FLXMRhyreg(type = type,
                             stv = stv,
                             type_cont = type_cont,
                             type_dich = type_dich,
                             variables_both = variables_both,
                             variables_cont = variables_cont,
                             variables_dich = variables_dich,
                             opt_method = opt_method,
                             optimizer = optimizer,
                             lower = lower,
                             upper = upper,
                             formula_type_classic = formula_type_classic))



    fit <- flexmix::flexmix(formula = formula, data = data, k = k, model = model, control = control)
    rm(counter, envir = the) # counter will be created during the M-step driver

    return(fit)

  }else{
    # latent = "cont" or "dich"

    # Prepare first step
    if(is.null(id_col)){
      stop("id_col needed")
    }

    idframe <- data.frame(id = data[,id_col],type)
    idcount <- as.data.frame(table(unique(idframe)))
    data <- data[!is.element(as.character(data[,id_col]),
                             as.character(idcount[idcount$Freq == 0, "id"])),]
    type <- idframe[!is.element(as.character(idframe[,"id"]),
                                as.character(idcount[idcount$Freq == 0, "id"])), "type"]

    if(any(idcount$Freq == 0)){
      miss <- idcount[idcount$Freq == 0, "id"]
      warning(paste0(id_col, paste(miss, collapse = ", "), " were removed, since they had only continuous or only dichotomous observations."))
    }

    # FIRST STEP: GET LATENT CLASSES
    if(latent == "cont"){
      data_cont <- data[type == type_cont,]
      assign("data", data_cont, envir = the)

      model <- list(FLXMRhyreg(type = type[type == type_cont],
                               stv = stv,
                               type_cont = type_cont,
                               type_dich = type_dich,
                               variables_both = variables_both,
                               variables_cont = variables_cont,
                               variables_dich = variables_dich,
                               opt_method = opt_method,
                               optimizer = optimizer,
                               lower = lower,
                               upper = upper,
                               formula_type_classic = formula_type_classic))


      mod <- flexmix::flexmix(formula = formula, data = data_cont, k = k, model = model, control = control)
      rm(counter, envir = the)



      # id_col must be something different than id if each observation should be classified
      data_cont$mod_comp <- mod@cluster

      data$roworder <- 1:nrow(data)
      data <- merge(data, unique(data_cont[, c(id_col, "mod_comp")]), by = id_col)
      data <- data[order(data$roworder), ]

      id_classes <- data_cont[,c(id_col, "mod_comp")]

    }

    if(latent == "dich"){
      data_dich <- data[type == type_dich,]
      assign("data", data_dich, envir = the)

      model <- list(FLXMRhyreg(type = type[type == type_dich],
                               stv = stv,
                               type_cont = type_cont,
                               type_dich = type_dich,
                               variables_both = variables_both,
                               variables_cont = variables_cont,
                               variables_dich = variables_dich,
                               opt_method = opt_method,
                               optimizer = optimizer,
                               lower = lower,
                               upper = upper,
                               formula_type_classic = formula_type_classic))


      mod <- flexmix::flexmix(formula = formula, data = data_dich, k = k, model = model, control = control)
      rm(counter, envir = the) # counter will be created during the M-step driver

      data_dich$mod_comp <- mod@cluster

      data$roworder <- 1:nrow(data)
      data <- merge(data, unique(data_dich[,c(id_col,"mod_comp")]), by = id_col)
      data <- data[order(data$roworder), ]

      # which id belongs to which class
      id_classes <- data_dich[,c(id_col, "mod_comp")]
    }

    # return only estimated classes without model new model coefficients
    if(classes_only == TRUE){
      return(id_classes)
    }


    # SECOND STEP: GET MODEL ESTIMATES

    # assign 1 to k in package environment to be used during M-step driver
    assign("k", 1, envir = the)

    data_list <- list()
    for(i in unique(mod@cluster)){
      data_list[[i]] <- data[data$mod_comp == i,]
    }


    mod_list <- lapply(data_list, function(xy){
      if(is.null(xy)){
        mod <- NULL
        warning( paste("One or more components are empty. Setting mod to NULL."))
      }else{
        model <- list(FLXMRhyreg(type = type[data$mod_comp == unique(xy$mod_comp)],
                                 stv = stv, # stv can be a list
                                 type_cont = type_cont,
                                 type_dich = type_dich,
                                 variables_both = variables_both,
                                 variables_cont = variables_cont,
                                 variables_dich = variables_dich,
                                 opt_method = opt_method,
                                 optimizer = optimizer,
                                 lower = lower,
                                 upper = upper,
                                 formula_type_classic = formula_type_classic))

        assign("data", xy, envir = the)

        mod <- flexmix::flexmix(formula = formula, data = xy, k = 1, model = model, control = control)
        rm(counter, envir = the) # counter will be created during the M-step driver
      }
      return(mod)
    })

    mod_list$id_classes <- unique(id_classes)
    return(mod_list)
  }
  rm(k, envir = the)
}





###############
### SUMMARY ###
###############


#' model summary for hyreg2 objects
#'
#' @description get model parameters of model generated by `hyreg2` or `hyreg2_het`
#'
#' @param object `modelobject` generated with [hyreg2()] or [hyreg2_het()]
#' @return `summary` object of [bbmle::mle2()] model,  Please note
#' that the outputs for `sigma` and `theta` are on a log-scale and have to be transformed using `exp()`to get the correct estimated values.
#'
#'
#' @author Svenja Elkenkamp
#' @examples
#'
#'formula <- y ~  -1 + x1 + x2 + x3 | id
#'k <- 1
#'stv <- setNames(c(0.2,0,1,1,1),c(colnames(simulated_data_norm)[3:5],c("sigma","theta")))
#'control = list(iter.max = 1000, verbose = 4)

#'rm(counter)

#'mod <- hyreg2(formula = formula,
#'                     data =  simulated_data_norm,
#'                     type =  simulated_data_norm$type,
#'                     stv = stv,
#'                     k = k,
#'                     type_cont = "TTO",
#'                     type_dich = "DCE_A",
#'                     opt_method = "L-BFGS-B",
#'                     control = control,
#'                     latent = "both",
#'                     id_col = "id"
#')
#'summary_hyreg2(mod)
#'
#' @importFrom bbmle mle2
#' @export


summary_hyreg2 <- function(object){
  if(is.list(object)){
    object <- object[1:(length(object) - 1)]
    out <- lapply(object, function(k){
      comp <- k@components
      out_list <- lapply(comp, function(j){bbmle::summary(j[[1]]@parameters[["fit_mle"]])})
    })
  }else{
    comp <- object@components
    out <- lapply(comp, function(j){bbmle::summary(j[[1]]@parameters[["fit_mle"]])})
  }
  return(out)
}


#########################

#' extract parameter estimates as named vector
#'
#' @description function to export coefficient values and names from a `model` fitted by `hyreg2` or `hyreg2_het`
#' These values can be used as `stv` for a new model with `k > 1`
#' @param mod `model`output from [hyreg2] oder [hyreg2_het]. If `latent` was `"cont"` or `"dich"` only one element of the output list can be used.
#' @param comp `charachter`, default `"Comp.1"`. `"Comp.x"` indicating values from which model `component` (`x`) should be exported
#' @return `named vector` of parameter estimates from `mod`. Can be used as `stv` for additional model estimations using
#' `hyreg2` or `hyreg2_het`
#'
#' @author Svenja Elkenkamp
#' @examples
#'formula <- y ~  -1 + x1 + x2 + x3 | id
#'
#'k <- 1
#'stv <- setNames(c(0.2,0,1,1,1),c(colnames(simulated_data_norm)[3:5],c("sigma","theta")))
#'control = list(iter.max = 1000, verbose = 4)

#'rm(counter)

#'mod <- hyreg2(formula = formula,
#'                     data =  simulated_data_norm,
#'                     type =  simulated_data_norm$type,
#'                     stv = stv,
#'                     k = k,
#'                     type_cont = "TTO",
#'                     type_dich = "DCE_A",
#'                     opt_method = "L-BFGS-B",
#'                     control = control,
#'                     latent = "both",
#'                     id_col = "id"
#')
#'new_stv <- get_stv(mod)
#'
#' # these new_stv can be used in an other estimation using hyreg2 as stv

#' @export

get_stv <- function(mod, comp = "Comp.1"){
  stv <- c(mod@components[[comp]][[1]]@parameters$coef,
           mod@components[[comp]][[1]]@parameters$sigma,
           mod@components[[comp]][[1]]@parameters$theta)
  return(stv)
}


##################################

##############################
### Desciption of datasets ###
##############################


#' simulated_data_norm
#'
#' @format ## `simulated_data_norm`
#' A simulated data frame with 600 rows and 9 columns, following a combination of normal and binomial distribution
#' \describe{
#'   \item{type}{type of data, "TTO" indicates normal distribution, "DCE_A" indicates binomial distribution}
#'   \item{y}{result of the formula y ~ x1 + x2 + x3}
#'   \item{x1, x2, x3}{random numbers from rnorm}
#'   \item{class}{original class of the data point}
#'   \item{id}{id number of observations to simulated different persons}
#'   \item{y_non}{result of the formula y ~ (x1 \* beta1 + x2 \* beta3) \* (x1 \* beta1 + x3 \* beta3)}
#'   \item{y_cens}{column y censored at 3}
#'   ...
#' }
#' @source {simulated with true parameter values:
#'  Class 1: sigma  = 1.0, theta = 5 and c(x1,x2,x3) = c(0.5, -0.3, 0.8)
#'  Class 2: sigma  = 0.5, theta = 2 and c(x1,x2,x3) = c(1.4, 2.3, -0.2)}

"simulated_data_norm"


#' simulated_data_mo
#'
#' @format ## `simulated_data_mo`
#' A simulated data frame with 480 rows and 9 columns, following a combination of normal and binomial distribution
#' \describe{
#'   \item{type}{type of data, "TTO" indicates normal distribution, "DCE_A" indicates binomial distribution}
#'   \item{y}{result of the formula y ~ -1 + mo2 + mo3 + mo4 + mo5}
#'   \item{mo2, mo3, mo4, mo5}{dummy variables}
#'   \item{class}{original class of the data point}
#'   \item{id}{id number of observations to simulated different persons}
#'   \item{y_cens}{column y censored at 0 (lower boundary)}
#'   ...
#' }
#' @source {simulated with true parameter values:
#'  Class 1: sigma  = 0.001, theta = 0.2 and c(mo2,mo3,mo4,mo5) = c(0.005, 0.01, 0.08, 0.1)
#'  Class 2: sigma  = 0.1, theta = 2 and c(mo2,mo3,mo4,mo5) = c(0.2, 0.4, 0.6, 0.8)}

"simulated_data_mo"





#'  simulated_data
#'
#' @format ## `simulated_data`
#' A simulated data frame with 480 rows and 25 columns, following a combination of normal and binomial distribution
#' \describe{
#'   \item{type}{type of data, "TTO" indicates normal distribution, "DCE_A" indicates binomial distribution}
#'   \item{y}{result of the formula y ~ -1 + mo2 + mo3 + ... + ad4 + ad5}
#'   \item{mo2, mo3, mo4, mo5,
#'          sc2, sc3, sc4, sc5,
#'          ua2, ua3, ua4, ua5,
#'          pd2, pd3, pd4, pd5,,
#'          ad2, ad3, ad4, ad5,}{dummy variables for EQ5D data simulation}
#'   \item{class}{original class of the data point}
#'   \item{id}{id number of observations to simulated different persons}
#'   \item{y_cens}{column y censored at 2 (upper boundary)}
#'   ...
#' }
#' @source {simulated with true parameter values:
#'  Class 1: sigma  = 0.02, theta = 2 and
#'   c(mo2,mo3,mo4,mo5) = c(0.001, 0.05, 0.08, 0.1),
#'   c(sc2,sc3,sc4,sc5) = c(0.01, 0.2, 0.36, 0.5),
#'   c(ua2,au3,ua4,ua5) = c(0.015, 0.25, 0.5, 0.8),
#'   c(pd2,pd3,pd4,pd5) = c(0.1, 0.3, 0.4, 0.6),
#'   c(ad2,ad3,ad4,ad5) = c(0.09, 0.19, 0.6, 0.7)
#'
#'  Class 2: sigma  = 0.1, theta = 3 and
#'   c(mo2,mo3,mo4,mo5) = c(0.2, 0.4, 0.6, 0.8),
#'   c(sc2,sc3,sc4,sc5) = c(0.1, 0.3, 0.4, 0.5),
#'   c(ua2,au3,ua4,ua5) = c(0.2, 0.25, 0.6, 0.7),
#'   c(pd2,pd3,pd4,pd5) = c(0.05, 0.2, 0.27, 0.8),
#'   c(ad2,ad3,ad4,ad5) = c(0.15, 0.35, 0.4, 0.65)
#' }
#'
 "simulated_data"

---
title: "hyreg2 basics"
output: 
  html_document:
    toc: true
    toc_depth: 3
date: "2025-12-12"
editor_options: 
  markdown: 
    wrap: sentence
vignette: >
  %\VignetteIndexEntry{hyreg2 basics}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: references.bib
---

<!-- rmarkdown::html_vignette -->

# The package in general

Our R package `hyreg2` allows users to estimate models on a combination of continuous and dichotomous data using a combined likelihood.
Latent classes can be detected using the EM-algorithm[@dempster1977].
The package was originally written to address preference heterogeneity or use a data-driven subgrouping of participants while estimating EQ-5D value sets[@devlin2022].
However, it can be applied to any situation in which a combined likelihood of continuous and dichotomous data should be maximized, e.g. as usual in environmental economics [@cameron1987].

**Functions**

The code for estimating models is based on the R package `flexmix`[@flexmix], which offers the option to write custom likelihood functions (M-step-drivers) for use during the M-step[@leisch2004][@grün2008].
For the `hyreg2` package, we wrote M-step-drivers that implement the hybrid model likelihood as described by Ramos-Goni [@ramos-goñi2017].
Additional wrapper functions `hyreg2` and `hyreg2_het` are included in the package, such that user do not have to input the drivers to `flexmix` themselves.

![](images/Aufbau_hyreg2.png)

The package supports both linear and non-linear functions while using uncensored or censored data.
Furthermore, heteroscedasticity can be taken into account for linear models using the `hyreg2_het` function.
It is also possible to control for different covariates on the continuous and dichotomous parts of the data.
Start values to be used can differ between the expected latent classes.
Additionally, `hyreg2` offers the option to estimate the latent classes either on both types of data (continuous and dichotomous) or only on one type.
Once a model has been estimated, the functions `summary_hyreg2`, `give_class`, and `plot_hyreg2` can be used to evaluate the results.

# Basic functionalities

When using `hyreg2` to estimate latent class models, it is essential to account for groups of observations.
Controlling for such dependencies is essential to ensure valid parameter estimation and inference.

To demonstrate the implications of neglecting to control for groups of observations, we first simulate a dataset with two latent classes.
Afterwards we estimate a model without controlling for groups and explain where the model fails.
At the end of this chapter, we will demonstrate how a good model fit can be achieved.

## Simulate Dataset

We demonstrate the basic functionalities using a part of the the simulated dataset `simulated_data_norm`, which is included in the `hyreg2` package.
The dataset consists of 7 variables, with 200 continuous and 100 dichotomous observations in each of the two latent classes, resulting in a total number of 600 observations.
The variables `x1`, `x2` and `x3` contain random draws from a standard normal distribution.
Based on these, outcome values `y` are generated according to the linear predictor

$x_1 * \beta_{x1} +x_2 *  \beta_{x2}+ x_3 * \beta_{x3} = Xb$.


For the continuous data, `y` values were generated directly using the function rnorm with `Xb` or `Xb_non` as the mean.
For the dichotomous data, the values of `Xb` are multiplied by a scaling parameter `theta` and subsequently simulated using the function `rbinom`.
The code for the simulation can be seen below.

```{r chunk-1, message=FALSE, warning=FALSE}
# simulate dataframe called simulated_data_norm

library(hyreg2)
library(flexmix)


### CLASS 1 ###

# Set parameters
set.seed(42)
n_samples_tto1 <- 200
n_samples_dce1 <- 100
sigma1 <- 1.0
theta1 <- 5
beta1 <- c(0.5, -0.3, 0.8)  # true values for beta

# simulate matrix
x_tto1 <- matrix(rnorm(n_samples_tto1 * length(beta1)), ncol = length(beta1))
x_dce1 <- matrix(rnorm(n_samples_dce1 * length(beta1)), ncol = length(beta1))



#Xb (linear predictor following formula xb = x1*beta1 + x2*beta2 + x3*beta3)
Xb_tto1 <- x_tto1 %*% beta1
Xb_dce1 <- (x_dce1 %*% beta1) * theta1


# generate y
y_tto1 <- rnorm(n_samples_tto1, mean = Xb_tto1, sd = sigma1)  # continuous outcomes
logistic_tmp1 <- 0.5 + 0.5 * tanh(Xb_dce1 / 2)
y_dce1 <- rbinom(n_samples_dce1, size = 1, prob = logistic_tmp1)  # binary outcomes



### dataset ###
data_tto1 <- data.frame(
  type = rep("TTO", n_samples_tto1),
  y = y_tto1,
  x1 = x_tto1[, 1],
  x2 = x_tto1[, 2],
  x3 = x_tto1[, 3]
)

data_dce1 <- data.frame(
  type = rep("DCE_A", n_samples_dce1),
  y = y_dce1,
  x1 = x_dce1[, 1],
  x2 = x_dce1[, 2],
  x3 = x_dce1[, 3]
)

simulated_data1 <- rbind(data_tto1, data_dce1)


### CLASS 2 ###

# Set parameters
set.seed(37)
n_samples_tto2 <- 200
n_samples_dce2 <- 100
sigma2 <- 0.5
theta2 <- 2
beta2 <- c(1.4, 2.3, -0.2)   # true values for beta


# simulate matrix
x_tto2 <- matrix(rnorm(n_samples_tto2 * length(beta2)), ncol = length(beta2))
x_dce2 <- matrix(rnorm(n_samples_dce2 * length(beta2)), ncol = length(beta2))


#  Xb (linear predictor following formula xb = x1*beta1 + x2*beta2 + x3*beta3)
Xb_tto2 <- x_tto2 %*% beta2
Xb_dce2 <- (x_dce2 %*% beta2) * theta2


# generate y
y_tto2 <- rnorm(n_samples_tto2, mean = Xb_tto2, sd = sigma2)  # continuous outcomes
logistic_tmp2 <- 0.5 + 0.5 * tanh(Xb_dce2 / 2)
y_dce2 <- rbinom(n_samples_dce2, size = 1, prob = logistic_tmp2)  # binary outcomes



### dataset ###
data_tto2 <- data.frame(
  type = rep("TTO", n_samples_tto2),
  y = y_tto2,
  x1 = x_tto2[, 1],
  x2 = x_tto2[, 2],
  x3 = x_tto2[, 3]
)

data_dce2 <- data.frame(
  type = rep("DCE_A", n_samples_dce2),
  y = y_dce2,
  x1 = x_dce2[, 1],
  x2 = x_dce2[, 2],
  x3 = x_dce2[, 3]
)

simulated_data2 <- rbind(data_tto2, data_dce2)


### DATASET with two classes ###
simulated_data1$class <- 1
simulated_data2$class <- 2

# set id numerbs
simulated_data1$id <- c(1:100, 1:100, 1:100)
simulated_data2$id <- c(101:200, 101:200, 101:200)


simulated_data_norm<- rbind(simulated_data1,simulated_data2)


# clean environment
rm(list = setdiff(ls(), "simulated_data_norm"))

```

The column `type` indicates whether a data point was generated according to a normal distribution (continuous data) or a binomial distribution (dichotomous data).
The column `id` specifies which continuous observations are linked with which dichotomous observations, and hence is a grouping variable.
For this dataset, each `id` is associated with two continuous observations and one dichotomous observation.
The column `class` contains the true underlying class memberships of the data.
This is visualized in figure 1 using the function `plot_hyreg2`.

```{r chunk-2, message=FALSE, warning=FALSE}

# visualize true classes from simulated_data_norm
p <- plot_hyreg2(data = simulated_data_norm,
           x = "id",
           y = "y",
           id_col = "id",
           class_df_model = simulated_data_norm[,c("id","class")],
           colors = c("#F8766D","#00BFC4"))
p <- p +  ggplot2::labs(title = "Figure 1")
p

```

The function `plot_hyreg2` expects a dataset (`data`) and two column names of this dataset as characters (`x` and `y`).
Additionally, the inputs `id_col` and `class_df_model` must be specified.
`id_col` must be the name of the grouping variable (as a character string) while `class_df_model` must be a dataframe containing two columns: the grouping variable (which must be named according to `id_col`) and the associated classes.
In this case, we already know which id is in which class, since we have simulated the dataset ourselves.
In real-world applications, users should instead use the `give_class` function to match each id to the class estimated by the model.
This is explained in further detail below.
`plot_hyreg2` returns an `ggplot`-object, hence additional `ggplot` specific parameters can be added, as demonstrated above.

## Estimation without further restrictions

First, we estimate a model without controlling for groups, i.e., we assign latent classes to each observation independently, without accounting for the fact that, in the simulated dataset, two continuous and one dichotomous observation belong together.

Since our goal is to identify two latent classes, we specify `k = 2`.
As we would like to estimate the mathematical model $x_1 * \beta_{x1} +x_2 *  \beta_{x2}+ x_3 * \beta_{x3} = y$, we specify `formula` in R as `y ~ -1 + x1 + x2 + x3`.
Starting values must be provided for all variables in formula, hence for `x1, x2` and `x3`.
In addition, starting values for the scaling parameter `theta` and the parameter `sigma` (from the continuous data) must be supplied.
These values must necessarily be named `“theta”` and `“sigma”`, otherwise the function will not recognize them.

The above formula does not include an intercept.
If the model to be estimated does contain an intercept or interaction terms, start values must be provided for these as well.
They must be named according to the column names of model.matrix(formula,data), e.g. "(Intercept)".

Now, we specify a named vector `stv`, which contains the assumed start values for `x1, x2, x3` and for `theta`and `sigma`.Furthermore, in `hyreg2` the input argument `type` must be specified.
In our dataset, the column indicating which datapoint was simulated from a normal distribution and which from a binomial distribution is simply called type with the entries `“TTO”` and `“DCE_A”`.
In this first example, we set `latent = “both”`, which implies that the latent classes are assumed to manifest in both types of data.

```{r chunk-3, message=FALSE, warning=FALSE, results='hide'}

set.seed(277)
formula <- y ~  -1 + x1 + x2 + x3
k <- 2
stv <- setNames(c(0.2,0,1,1,1),c("x1","x2","x3","sigma","theta"))
control <- list(iter.max = 1000, verbose = 4)

mod <- hyreg2(formula = formula,
                     data =  simulated_data_norm,
                     type =  simulated_data_norm$type,
                     stv = stv,
                     k = k,
                     type_cont = "TTO",
                     type_dich = "DCE_A",
                     opt_method = "L-BFGS-B",
                     control = control,
                     latent = "both"
)

```

The function `summary_hyreg2` can be used to display the estimated parameters for both model classes.
Please note, that the outputs for `sigma` and `theta` are on a log-scale and have to be transformed using `exp()`to get the correct estimated values.

```{r chunk-4, message=FALSE, warning=FALSE}

summary_hyreg2(mod)

```

We visualize the estimated values using the function `plot_hyreg2`.
The argument `id_col` refers to the grouping column.
If the model was estimated without grouping (as in this case), the row names/numbers should simply be provided.
The input `class_df_model` requires a data frame with two columns, indicating which group (or observation) from the data has been assigned to which class by the model.
Such a data frame can easily be generated using the function `give_class`.
Again, if the model was estimated without controlling for groups, the row names of the dataset should be used as the input for `id_col` in the function `give_class`.

```{r chunk-5, message=FALSE, warning=FALSE}

# if model was estimated without grouping variable
simulated_data_norm$observation <- rownames(simulated_data_norm)


p <- plot_hyreg2(data = simulated_data_norm,
           x = "id",
           y = "y",
           id_col = "observation",
           class_df_model = give_class(data = simulated_data_norm,
                                 model = mod,
                                 id_col = "observation"))
p <- p +  ggplot2::labs(title = "Figure 2")
p

```

This plot shows both types of observations (continuous and dichotomous).
There seem to be two lines of mostly blue (class 2) datapoints.
These are the datapoints corresponding to the dichotomous data.
It seems that the model (erroneously!) classified the data by datatypes, rather than by latent classes.

Using `plot_hyreg2`, it is also possible to display only the continuous data or only the dichotomous data.
For this purpose, the argument `type_to_plot` is used.
This argument expects a `list` with two elements: the first element contains the name of the column used to differentiate between the data types (as a character string), and the second element specifies the value associated with the desired type of data to be included in the plot.
This works analogously to `type` and `type_cont` / `type_dich` in the `hyreg2` input.

```{r chunk-6, message=FALSE, warning=FALSE}

# only continuous datapoints 
p <- plot_hyreg2(data = simulated_data_norm,
           x = "id",
           y = "y",
           id_col = "observation",
           class_df_model = give_class(data = simulated_data_norm,
                                 model = mod,
                                 id_col = "observation"),
           type_to_plot = list("type","TTO"))
p <- p +  ggplot2::labs(title = "Figure 3")
p


```

```{r chunk-7, message=FALSE, warning=FALSE}

# only dichotomous datapoints
p <- plot_hyreg2(data = simulated_data_norm,
           x = "id",
           y = "y",
           id_col = "observation",
           class_df_model = give_class(data = simulated_data_norm,
                                 model = mod,
                                 id_col = "observation"),
           type_to_plot = list("type","DCE_A"))
p <- p +  ggplot2::labs(title = "Figure 4")
p


```

Figure 3 shows only the continuous datapoints while figure 4 shows only the dichotomous ones.
It can be seen that the dichotomous observations of the lower id numbers were often classified to class 2 while the continuous observations of the lower id numbers often were classified to class 1.
Additionally, the visualizations suggest that a considerable proportion of the dichotomous data is concentrated in class 2, whereas the majority of continuous data are assigned to class 1.
Even though the estimates of the model parameters for class 1 come quite close to the true parameters, the classification overall is far away from the true underlying classes, since only 58 % of the datapoints were classified correctly.
Moreover, it can be useful to have a look at the distribution of data types across the two latent classes.

```{r chunk-8, message=FALSE, warning=FALSE}

# ratio of correct classified datapoints
(sum(mod@cluster == simulated_data_norm$class))/dim(simulated_data_norm)[1]



# cross table of observation types and assigned classes
proof <- merge(simulated_data_norm,
                 give_class(simulated_data_norm,mod,"observation"), by = "observation")
table(proof$type,proof$mod_comp)




```

The model classifies more than 75% of the dichotomous data into class 2 and less than 25% into class 1.
By contrast, more than 60% of the continuous data points are assigned to class 1.
Overall, only 14% of the observations in class 1 are dichotomous, whereas in class 2 their proportion exceeds 50%.

All in all, this shows that it is necessary to control for groups of observations during the estimation process.
Otherwise, the model tends to differentiate between dichotomous and continuous data.
However, this represents an undesired behavior, as we would expect both classes to exhibit the same proportion of continuous and dichotomous data.

## Classifying individuals not observations

Including a grouping variable in the formula ensures that the class probabilities in the E-step of the EM-algorithm (via `flexmix`) are estimated per group rather than per observation.
This can be done by adding `"| group_variable_name"` to `formula`.
Further details can be found in Gruen and Leisch.
This specification has no impact on the likelihood function used in the M-step.
The model estimates change solely because groups, rather than each observation, are classified.

```{r chunk-9, message=FALSE, warning=FALSE, results='hide'}
set.seed(227)
formula <- y ~  -1 + x1 + x2 + x3 | id
k <- 2
stv <- setNames(c(0.2,0,1,1,1),c("x1","x2","x3","sigma","theta"))
control <- list(iter.max = 1000, verbose = 4)

mod_id <- hyreg2(formula = formula,
                     data =  simulated_data_norm,
                     type =  simulated_data_norm$type,
                     stv = stv,
                     k = k,
                     type_cont = "TTO",
                     type_dich = "DCE_A",
                     opt_method = "L-BFGS-B",
                     control = control,
                     latent = "both"
)

```

```{r chunk-9a, message=FALSE, warning=FALSE}

summary_hyreg2(mod_id)


```

The model estimates look worse than in the model without controlling for groups.
Nevertheless, we will examine the classification achieved by the model in the following plot.

```{r chunk-10, message=FALSE, warning=FALSE}

p <- plot_hyreg2(data = simulated_data_norm,
           x ="id",
           y = "y",
           id_col ="id",
           class_df_model = unique(give_class(data = simulated_data_norm,
                                        model = mod_id,
                                        id_col = "id")))
p <- p +  ggplot2::labs(title = "Figure 5")
p


```

It can be seen that most of the dichotomous and continuous data points with lower id numbers were assigned to class 1.
For higher id numbers, an assignment to class 2 would be expected.
Although this assignment occurred in many cases, several data points with higher id numbers were nevertheless incorrectly classified to class 1.
An evaluation of the classification indicates that 74% of the data points were correctly assigned, representing a clear improvement compared to the model without control for grouping.
An examination of the distribution further reveals that in both classes, approximately 33% of the data are dichotomous and about 67% are continuous.
This pattern corresponds to the data-generating process used in the simulation.
However, the resulting classes differ in total size.
Overall, too many datapoints have been assigned to class 1.

```{r chunk-13, message=FALSE, warning=FALSE}

# ratio of correct classified datapoints
(sum(mod_id@cluster == simulated_data_norm$class))/dim(simulated_data_norm)[1]


# cross table of observation types and assigned classes

proof <- merge(simulated_data_norm,
                 unique(give_class(simulated_data_norm,mod_id,"id")), by = "id")
table(proof$type,proof$mod_comp)



```

In the dichotomous data, only the values 0 and 1 can be distinguished and the simulated dataset contains only one dichotomous datapoint for each id number.
Hence, no patterns can be detected based on these dataponints.
Therefore, it can be concluded, that the data can primarily be distinguished based on the continuous data points.
Lower id numbers in general exhibit less variability compared to higher id numbers.

This outcome is consistent with our expectations, as the data were simulated accordingly: The latent classes were determine only by the continuous data, where the beta coefficients were explicitly defined.
The subsequent combination of these beta values with theta then produced the dichotomous data points.

## Estimating latent classes only on continuous data

If it is assumed (as in this simulation) that the latent classes are determined only by one of the two data types, the input variable `latent` allows this to be taken into account during estimation.
It is possible to estimate latent classes only on continuous data (`latent = “cont”`) or only on dichotomous data (`latent = “dich”`).
If one of these options is chosen, it is not possible to estimate models without controlling for groups.
Additionally, the input variable `id_col` must be specified; it gives the name of the grouping variable as character.
Latent classes are then determined in a first step based only on one type of data.
In a second step, these classes are used to estimate the model parameters on both types of data.

Here, we want to account for preference heterogeneity in the continuous data part, and hence we set `latent = “cont”`.

```{r chunk-14, message=FALSE, warning=FALSE, results='hide'}
set.seed(227)
formula <- y ~  -1 + x1 + x2 + x3 | id
k <- 2
stv <- setNames(c(0.2,0,1,1,1),c("x1","x2","x3","sigma","theta"))
control <- list(iter.max = 1000, verbose = 4)

mod_cont <- hyreg2(formula = formula,
                     data =  simulated_data_norm,
                     type =  simulated_data_norm$type,
                     stv = stv,
                     k = k,
                     type_cont = "TTO",
                     type_dich = "DCE_A",
                     opt_method = "L-BFGS-B",
                     control = control,
                     latent = "cont",
                     id_col = "id"
)
```

The output object of `hyreg2` in this case is a list of length k+1 (k models plus one data frame called "id_classes", which contains the estimated group memberships from the first estimation step).
The functions `summary_hyreg2` and `plot_hyreg2` can still be applied without restrictions, as they automatically recognize whether the input is a single model or a list of models.

```{r chunk-14a, message=FALSE, warning=FALSE}

summary_hyreg2(mod_cont)

```

The estimates of this model nearly capture the true parameter values, especially for `x1` in both classes.
`x2` and `x3` are less precise, although the results are still acceptable.
Again, we have a look at the overall classification.

```{r chunk-15, message=FALSE, warning=FALSE}

p <- plot_hyreg2(data = simulated_data_norm,
           x ="id",
           y = "y",
           id_col ="id",
           class_df_model = give_class(data = simulated_data_norm,
                                model = mod_cont,
                                id_col = "id"))
p <- p +  ggplot2::labs(title = "Figure 6")
p


```

```{r chunk-16, message=FALSE, warning=FALSE}

# ratio of correct classified datapoints
proof <- merge(unique(simulated_data_norm[,c("id","class")]),mod_cont[["id_classes"]], by = "id")
sum((proof$class == proof$mod_comp)/dim(proof)[1])


# cross table of observation types and assigned classes

proof <- merge(simulated_data_norm,
                mod_cont[[k+1]], by = "id")
table(proof$type,proof$mod_comp)



```

Assessing the ratio of correct classification reveals that 95 % of the observations are now classified to the correct class.
In addition, the data are evenly distributed across classes and data types, as it was generated during the simulation.

Hence, using the input parameter `latent = “cont”` under control for the grouping variable id leads to very good results, since the true underlying classes could be identified and the estimated parameters come close to the true beta values from the data simulation.

# Advanced functionalities

In this section, we present the additional functionalities of the package.
We demonstrate how different types of input must be specified in order to estimate well-performing models.
At this point, we refrain from providing detailed interpretations of the obtained results.

## Estimating models using censored data

The function `hyreg2` can be applied to censored data by specifying the parameters `lower` and/or `upper`.
We simulate a new column `y_cens` in our dataset simulated_data_norm with censoring at `y = 3`.
This censoring can be incorporated in the model estimation by setting the parameter `upper = 3`.
Equivalent parameter `lower` can be set to a number, where data have lower bounds.

```{r chunk-18, message=FALSE, warning=FALSE, results='hide'}

# upper bound for simulated_data_norm$y at 3
# new variable y_cens

for(i in 1:length(simulated_data_norm$y)){
  if(simulated_data_norm$y[i] <= 3){
    simulated_data_norm$y_cens[i] <- simulated_data_norm$y[i]
  }else{
    simulated_data_norm$y_cens[i] <- 3
  }
}


set.seed(227)
formula <- y_cens ~  -1 + x1 + x2 + x3 | id
k <- 2
stv <- setNames(c(0.2,0,1,1,1),c("x1","x2","x3","sigma","theta"))
control <- list(iter.max = 1000, verbose = 4)

mod_cens <- hyreg2(formula = formula,
                     data =  simulated_data_norm,
                     type =  simulated_data_norm$type,
                     stv = stv,
                     k = k,
                     type_cont = "TTO",
                     type_dich = "DCE_A",
                     opt_method = "L-BFGS-B",
                     control = control,
                     upper = 3,
                     latent = "cont",
                     id_col = "id"
)

```

We can inspect the results using `summary_hyreg2` and `plot_hyreg2`.

```{r chunk-19, message=FALSE, warning=FALSE, results='hide', fig.show='hide'}
summary_hyreg2(mod_cens)


plot_hyreg2(data = simulated_data_norm,
           x ="id",
           y = "y_cens",
           id_col = "id",
           class_df_model = give_class(data = simulated_data_norm,
                                 model = mod_cens,
                                 id_col = "id"))



```

## Providing start values as a list

In some situations, it may be useful to specify different starting values for the different latent classes.
This can potentially shorten or improve the estimation process, for instance, when local rather than global maxima are found during maximum likelihood estimation.
For this purpose, the input parameter `stv` can simply be defined as a `list` consisting of k named vectors.

```{r chunk-22, message=FALSE, warning=FALSE, results='hide'}
set.seed(227)
formula <- y ~  -1 + x1 + x2 + x3 | id
k <- 2
stv1 <- setNames(c(0.2,0,1,1,1),c("x1","x2","x3","sigma","theta"))
stv2 <- setNames(c(1,2,0.5,1,1),c("x1","x2","x3","sigma","theta"))
control <- list(iter.max = 1000, verbose = 4)

mod_stvl<- hyreg2(formula = formula,
                     data =  simulated_data_norm,
                     type =  simulated_data_norm$type,
                     stv = list(stv1,stv2),
                     k = k,
                     type_cont = "TTO",
                     type_dich = "DCE_A",
                     opt_method = "L-BFGS-B",
                     control = control,
                     latent = "cont",
                     id_col = "id"
)

summary_hyreg2(mod_stvl)

```

## Accounting for heteroscedasticity using start values from previous model estimates

**Function get_stv**

In some cases, it can be useful to use the estimated values from a simpler model as starting values for a subsequent model.
For this purpose, the function `get_stv` can be used.
Its input must be a previously estimated model.
If this model was estimated with `latent = "both"`, it is sufficient to provide the model name as input.
Additionally, the parameter `comp` can be used to specify from which class the parameters should be extracted, for example: `get_stv(mod = mod_id, comp = "Comp.1")`.
By default, the first class/component is used.
If the model to be used was estimated with `latent = "cont"` or `latent = "dich"`, the output of `hyreg2`is a list.
In this case, one of the k models from the output must be explicitly selected as input for `get_stv`, for example: `get_stv(mod = mod_cont[[1]])`.
Additionally, in this case, the input parameter `comp` must always be `"Comp.1"`, since the model component to be used is selected by specifying the list element.

**Accounting for heteroscedasticity**

The `hyreg2` package includes an additional function and an M-step driver that allow controlling for heteroscedasticity by estimating a separate model for sigma.
The corresponding function is called `hyreg2_het`.
This function supports all functionalities of `hyreg2` (except `formula_type_classic = FALSE`) and can therefore be used in an equivalent manner.
To account for heteroscedasticity in the data, an additional formula called `formula_sigma` and an additional vector of starting values for this formula (`stv_sigma`) can be specified.
The provided `formula_sigma` must be linear and the vector `stv_sigma` must contain start values for all parameters used in `formula_sigma`.
It is important to note that, when using `hyreg2_het`, neither `stv` nor `stv_sigma` are allowed to include a start value called `"sigma"`, because sigma is estimated with its own formula (in contrast to `hyreg2`, where `"sigma"` must always be specified in `stv`).

```{r chunk-24, message=FALSE, warning=FALSE, results='hide'}

set.seed(227)
formula <- y ~  -1 + x1 + x2 + x3 | id
formula_sigma <- y ~ x2 + x3
k <- 2
stv1 <- get_stv(mod_cont[[1]])[-4]  # -4 since "sigma" is on fourth place in return vector
stv2 <- get_stv(mod_cont[[2]])[-4]  # -4 since "sigma" is on fourth place in return vector
stv_sigma <- setNames(rep(0.1,3), c("x2","x3","(Intercept)"))
control <- list(iter.max = 1000, verbose = 4)


mod_het <- hyreg2_het(formula = formula,
                     formula_sigma = formula_sigma,
                     data =  simulated_data_norm,
                     type =  simulated_data_norm$type,
                     stv = list(stv1,stv2),
                     stv_sigma = stv_sigma,
                     k = k,
                     type_cont = "TTO",
                     type_dich = "DCE_A",
                     opt_method = "L-BFGS-B",
                     control = control,
                     latent = "cont",
                     id_col = "id"
)
```

```{r chunk-24a, message=FALSE, warning=FALSE}
summary_hyreg2(mod_het)

```

The estimates for sigma can be identified in the model output by the ending `"_h"`.

## Estimating models using partial coefficients

It is possible to specify partial coefficients, which are used only on continuous or only on dichotomous data.

*Example:*

Suppose different models should be specified for continuous and dichotomous data.
For example:

-   Continuous data: `y ~ x1 + x3`

-   Dichotomous data: `y ~ x1 + x2`

The `formula` input to `hyreg2` must then include all parameters that occur in either (i.e., in at least one) model:

`y ~ x1 + x2 + x3`

The assignment of parameters to data types is then achieved via the input arguments:

`variables_both = ⁠“x1”`⁠, `variables_cont = ⁠“x3”`⁠ and `variables_dich = ⁠“x2”`⁠.

Every variable included in the provided `formula` (except the grouping variable ) must appear in exactly one of these vectors.
One of the variables\_ vectors can also be `NULL`, if no variables should be used exclusively on this type of the data.

```{r chunk-23, message=FALSE, warning=FALSE, results='hide'}
set.seed(227)
formula <- y ~  -1 + x1 + x2 + x3 | id
k <- 2
stv <- setNames(c(0.2,0,1,1,1),c(colnames(simulated_data_norm)[3:5],c("sigma","theta")))
control <- list(iter.max = 1000, verbose = 4)

mod_partialc <- hyreg2(formula = formula,
                     data =  simulated_data_norm,
                     type =  simulated_data_norm$type,
                     stv = stv,
                     k = k,
                     type_cont = "TTO",
                     type_dich = "DCE_A",
                     opt_method = "L-BFGS-B",
                     control = control,
                     latent = "cont",
                     id_col = "id",
                     variables_both = "x1",
                     variables_cont = "x3",
                     variables_dich = "x2"
)

summary_hyreg2(mod_partialc)

```

## Estimating models using non-classic formulas

All inputs and functionalities that have been considered so far are used with classic R formulas and the default setting (`TRUE`) for the input `formula_type_classic`.
However, with this setting, it is not possible to estimate non-linear formulas or the 8-parameter model for EQ-5D data[@devlin2022] .
Therefore, in `hyreg2`, it is possible to specify model formulas with variables and parameters (non-classic formulas).
For example, if a model should be estimated for the mathematical equation (1) $(x_1 * \beta_1 + x_2 * \beta_2) + (x_1 * \beta_1 + x_3 * \beta_3) = y$ or (2) $\frac{1}{e^{x_1 * \beta_{1} + x_2 *  \beta_{2}}} = y$, this must be specified as a formula with additional parameters provided, which are not column names in the dataset: (1)` formula <- y ~ (x1 * beta1 + x2 * beta2 ) * (x1 *beta1 + x3 * beta3)` or (2)`formula = y ~ 1/exp(x1 * beta1 + x2 * beta2)`, where the `beta`s represent the parameters to be estimated and the`x`s are column names from the dataset.

Additionally, `formula_type_classic` must be set to `FALSE`.
When estimating an intercept, the formula must explicitly include a parameter named `"INTERCEPT"`(without a corresponding variable from the dataset), e.g. `formula = y ~  1/exp(INTERCEPT + x1 * beta1 + x2 * beta2)`

We simulate a new column called `y_non` for the dataset simulated_data_norm and show how to estimate a model using non-classic formulas.

.

```{r chunk-26, message=FALSE, warning=FALSE}

# simulate vector y_non and add this to simulated_data_norm

#library(hyreg2)

### CLASS 1 ###

# Set parameters (same as in simulation above)
set.seed(42)
n_samples_tto1 <- 200
n_samples_dce1 <- 100
sigma1 <- 1.0
theta1 <- 5
beta_non1 <- c(0.5, -0.3, 0.8)  # true values for beta

# simulate matrix
x_tto1 <- matrix(rnorm(n_samples_tto1 * length(beta_non1)), ncol = length(beta_non1))
x_dce1 <- matrix(rnorm(n_samples_dce1 * length(beta_non1)), ncol = length(beta_non1))


### NON-classic ###
# x1,x2 and x3 are columnnames of the dataset, beta1, beta2 and beta3 are the parameters to be estimated
formula1 <- y ~ (x1 * beta1 + x2 * beta2 ) * (x1 *beta1 + x3 * beta3)
x_tto_non1 <- data.frame(x1 = x_tto1[,1], x2 = x_tto1[,2], x3 = x_tto1[,3])
x_dce_non1 <- data.frame(x1 = x_dce1[,1], x2 = x_dce1[,2], x3 = x_dce1[,3])
beta_non1 <- setNames(beta_non1[1:3], c("beta1","beta2","beta3"))

Xb_tto_non1 <- hyreg2:::eval_formula_non(formula1, x_tto_non1, beta_non1)
Xb_dce_non1 <- (hyreg2:::eval_formula_non(formula1, x_dce_non1, beta_non1)) * theta1


# generate y_non
y_tto_non1 <- rnorm(n_samples_tto1, mean = Xb_tto_non1, sd = sigma1)  # continuous outcomes
logistic_tmp_non1 <- 0.5 + 0.5 * tanh(Xb_dce_non1 / 2)
y_dce_non1 <- rbinom(n_samples_dce1, size = 1, prob = logistic_tmp_non1)  # binary outcomes



### CLASS 2 ###

# Set parameters (same as in simulation above)
set.seed(37)
n_samples_tto2 <- 200
n_samples_dce2 <- 100
sigma2 <- 0.5
theta2 <- 2
beta_non2 <- c(1.4, 2.3, -0.2)   # true values for beta


# simulate matrix
x_tto2 <- matrix(rnorm(n_samples_tto2 * length(beta_non2)), ncol = length(beta_non2))
x_dce2 <- matrix(rnorm(n_samples_dce2 * length(beta_non2)), ncol = length(beta_non2))



### NON-classic ###
# x1,x2 and x3 are columnnames of the dataset, beta1, beta2 and beta3 are the parameters to be estimated
formula2 <- y  ~ (x1 * beta1 + x2 * beta2 ) * (x1 * beta1 + x3 * beta3)
x_tto_non2 <- data.frame(x1 = x_tto2[,1], x2 = x_tto2[,2], x3 = x_tto2[,3])
x_dce_non2 <- data.frame(x1 = x_dce2[,1], x2 = x_dce2[,2], x3 = x_dce2[,3])
beta_non2 <- setNames(beta_non2[1:3], c("beta1","beta2","beta3"))

Xb_tto_non2 <- hyreg2:::eval_formula_non(formula2, x_tto_non2, beta_non2)
Xb_dce_non2 <- (hyreg2:::eval_formula_non(formula2, x_dce_non2, beta_non2)) * theta2


# generate y_non
y_tto_non2 <- rnorm(n_samples_tto2, mean = Xb_tto_non2, sd = sigma2)  # continuous outcomes
logistic_tmp_non2 <- 0.5 + 0.5 * tanh(Xb_dce_non2 / 2)
y_dce_non2 <- rbinom(n_samples_dce2, size = 1, prob = logistic_tmp_non2)  # binary outcomes


simulated_data_norm$y_non <- c(y_tto_non1,y_dce_non1,y_tto_non2,y_dce_non2)

# clean environment
rm(n_samples_tto1,
   n_samples_dce1,
   sigma1,
   theta1,
   beta_non1,
   n_samples_tto2,
   n_samples_dce2,
   sigma2,
   theta2,
   beta_non2,
   logistic_tmp1,
   logistic_tmp2,
   logistic_tmp_non1,
   logistic_tmp_non2,
   x_tto_non1,
   x_tto_non2,
   x_dce_non1,
   x_dce_non2,
   Xb_tto_non1,
   Xb_tto_non2,
   Xb_dce_non1,
   Xb_dce_non2,
   x_dce1,
   x_dce2,
   x_tto1,
   x_tto2,
   y_dce_non1,
   y_dce_non2,
   y_tto_non1,
   y_tto_non2,
   formula1,
   formula2
)

```

If `formula_type_classic = FALSE`, the provided input for `stv` must contain the parameter names (e.g. beta1) and not the variable names (e.g. x1) from the dataset.
All functionalities described above (expect accounting for heteroscedasticity) can be used with variables-and-parameters-formulas as well.
If partial coefficients should be used, the provided variables_vectors must contain the names of the parameters and not the variable names from the dataset.

```{r chunk-25, message=FALSE, warning=FALSE, results='hide'}
set.seed(259)
formula <- y_non ~ (x1 * beta1 + x2 * beta2 ) * (x1 * beta1 + x3 * beta3)| id
k <- 2
stv <- setNames(c(0.2,0,1,1,1),c("beta1","beta2","beta3","sigma","theta"))
control <- list(iter.max = 1000, verbose = 4)

mod_non <- hyreg2(formula = formula,
                     data =  simulated_data_norm,
                     type =  simulated_data_norm$type,
                     stv = stv,
                     k = k,
                     type_cont = "TTO",
                     type_dich = "DCE_A",
                     opt_method = "L-BFGS-B",
                     control = control,
                     latent = "cont",
                     id_col = "id",
                     formula_type_classic = FALSE
)

```

The output can be displayed using `summary_hyreg2` and the classification can be inspected using `plot_hyreg2`.

```{r chunk-21, message=FALSE, warning=FALSE, results='hide'}

summary_hyreg2(mod_non)

p <- plot_hyreg2(data = simulated_data_norm, 
           x ="id",
           y = "y_non",
           id_col = "id",
           class_df_model = give_class(data = simulated_data_norm,
                                 model = mod_non,
                                 id_col = "id"))
p <- p +  ggplot2::labs(title = "Figure 7")
p


```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{simulated_data_mo}
\alias{simulated_data_mo}
\title{simulated_data_mo}
\format{
\subsection{\code{simulated_data_mo}}{

A simulated data frame with 480 rows and 9 columns, following a combination of normal and binomial distribution
\describe{
\item{type}{type of data, "TTO" indicates normal distribution, "DCE_A" indicates binomial distribution}
\item{y}{result of the formula y ~ -1 + mo2 + mo3 + mo4 + mo5}
\item{mo2, mo3, mo4, mo5}{dummy variables}
\item{class}{original class of the data point}
\item{id}{id number of observations to simulated different persons}
\item{y_cens}{column y censored at 0 (lower boundary)}
...
}
}
}
\source{
{simulated with true parameter values:
Class 1: sigma  = 0.001, theta = 0.2 and c(mo2,mo3,mo4,mo5) = c(0.005, 0.01, 0.08, 0.1)
Class 2: sigma  = 0.1, theta = 2 and c(mo2,mo3,mo4,mo5) = c(0.2, 0.4, 0.6, 0.8)}
}
\usage{
simulated_data_mo
}
\description{
simulated_data_mo
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hyreg2_het.R
\name{hyreg2_het}
\alias{hyreg2_het}
\title{Estimating hybrid models accounting for heteroscedasticity in continuous data}
\usage{
hyreg2_het(
  formula,
  formula_sigma = NULL,
  data,
  type,
  type_cont,
  type_dich,
  k = 1,
  control = NULL,
  stv = NULL,
  stv_sigma = NULL,
  offset = NULL,
  opt_method = "BFGS",
  optimizer = "optim",
  lower = -Inf,
  upper = Inf,
  latent = "both",
  id_col = NULL,
  classes_only = FALSE,
  variables_both = NULL,
  variables_dich = NULL,
  variables_cont = NULL,
  ...
)
}
\arguments{
\item{formula}{linear model \code{formula}. Using \verb{|xg} will include a grouping variable \code{xg}. see Details.}

\item{formula_sigma}{linear \code{formula} linear formula for sigma estimation. If \code{formula_sigma} is not provided,
\code{formula} (excluding any grouping variables) is used by default, see Details}

\item{data}{a \code{data.frame} containing the data. see Details.}

\item{type}{either the name of the column in \code{data} containing an indicator of whether an
observation is continuous or dichotomous (as \code{character}), or a \code{vector} containing the indicator.
see Details.}

\item{type_cont}{value of \code{type} referring to continuous data. see Details.}

\item{type_dich}{Value of \code{type} referring to dichotomous data. see Details.}

\item{k}{\code{numeric}. Number of latent classes to be estimated via \code{\link[flexmix:flexmix]{flexmix::flexmix()}}.}

\item{control}{control list for \code{\link[flexmix:flexmix]{flexmix::flexmix()}}.}

\item{stv}{\verb{named vector} or \code{list} of named vectors containing start values for all coefficients
formula, including sigma and theta, see Details}

\item{stv_sigma}{\verb{named vector} with start values for sigma estimation. Names must correspond to the variables
as given in \code{formula_sigma}, see Details}

\item{offset}{offset as in \code{\link[flexmix:flexmix]{flexmix::flexmix()}}}

\item{opt_method}{\code{charachter}, optimization method to be used in optimizer, default \code{"BFGS"}}

\item{optimizer}{\code{charachter},optimizer to be used in \code{\link[bbmle:mle2]{bbmle::mle2()}}, default \code{"optim"}}

\item{lower}{\code{numeric}, lower bound for censored data, default \code{-INF}. If this is used, \code{opt_method} must be set to \code{"L-BFGS-B"},}

\item{upper}{\code{numeric}, upper bound for censored data, default \code{INF}. If this is used, \code{opt_method} must be set to \code{"L-BFGS-B"},}

\item{latent}{\code{charachter},data type to use in component identification, must be one of \code{"both"}, \code{"cont"} or \code{"dich"},
default  \verb{“both”},  see Details}

\item{id_col}{\code{character}, name of the grouping variable, only needed if \verb{latent != "both”}, see Details}

\item{classes_only}{\code{logical}, default \code{FALSE}, indicates whether the function should perform only
classification, rather than both classification and model estimation, only possible for \code{latent != "both"},
see Datails}

\item{variables_both}{\verb{character vector}; variables to be fitted on both continuous and dichotomous data.
If not specified, all variables from \code{formula} are used. If provided and not all variables from \code{formula}
are included, \code{variables_cont} and \code{variables_dich} must be provided as well, while one of them can be \code{NULL},
see Details.}

\item{variables_dich}{\code{character} vector; variables to be fitted only on dichotomous data, if provided,
\code{variables_both} and \code{variables_cont} must be provided as well.}

\item{variables_cont}{\code{character} vector; variables to be fitted only on continous data. If provided,
\code{variables_both} and \code{variables_dich} must be provided as well.}

\item{...}{additional arguments for \code{\link[flexmix:flexmix]{flexmix::flexmix()}} or \code{\link[bbmle:mle2]{bbmle::mle2()}}}
}
\value{
model object of type flemix, coefficients named ..._h are coefficients for heteroscedasticity
}
\description{
Estimation of hybrid model using continuous and dichotomous data e.g. EQ-5D data
}
\details{
see details of different inputs listed below
}
\section{formula}{

a classic R formula of the form \verb{y ~ x1 + x2 + …} should be provided.
Additionally, it is possible to include a grouping variable for repeated measures by using
\verb{“| xg”} where \code{xg} is the column containing the group-memberships. The resulting formula will look
like this:  \verb{y ~ x1 + x2 +… | xg}.  In \code{flexmix}, this is called the concomitant variable specification:
the  model is fit conditional on grouping, so that all observations with the same group are treated
as belonging together when computing likelihood contributions. One possible grouping variable can be
an id number to identify answers by the same participants. We highly recommend using a grouping variable,
since otherwise the algorithm for k = 2 tends to classify all continuous data into one estimated class
and all dichotomous data into the other.
}

\section{data}{

a dataframe having the following columns: all independent variables (x)
and the dependent variable y used in \code{formula}, one column for the grouping variable xg if grouping
should be used, e.g. id numbers of participants with repeated measurements, one column indicating
if the observations belongs to continuous or dichotomous data with the entries \code{type_cont}
and \code{type_dich} (e.g., for a column called \code{"type"} with the entries "TTO" for continuous datapoints
and "DCE" for dichotomous datapoints, \code{type_cont} will be "TTO" and \code{type_dich} will be "DCE").
One row should match one observation (one datapoint).
}

\section{start values (stv)}{

if the same start values \code{stv} are to be used for all latent classes,
the given start values must be a \verb{named vector}. Otherwise (if different start values are assumed for
each latent class), a \code{list} of named vectors should be used . In this case, there must be one entry
in the list for each latent class.  Each start value vector must include start values for sigma and
theta. Currently, it is necessary to use the names \code{"sigma"} and \code{"theta"} for these values.
If users are unsure for which variables start values must be provided, this can be checked by
calling \code{colnames(model.matrix(formula,data))}. In this call, the \code{formula} should not include the
grouping variable.
}

\section{formula_sigma, stv_sigma}{

To account for heteroscedasticity in the data, an additional formula \code{formula_sigma} and an additional
vector of starting values for this formula (\code{stv_sigma}) can be specified.
The provided \code{formula_sigma} must be linear and the vector \code{stv_sigma} must contain start values for
all parameters used in the formula. If neither \code{formula_sigma} nor \code{stv_sigma} are provided, the same
inputs as for \code{formula} (without controlling for groups) and \code{stv} (without sigma) are used.
The estimates for \code{sigma} can be identified in the model output by the ending \code{"_h"}. It is important to note
that, when using \code{hyreg2_het}, neither \code{stv} nor \code{stv_sigma} are allowed to include \code{sigma},
because \code{sigma} is estimated with its own formula (in contrast to \code{hyreg2}, where \code{sigma} must always be
specified in \code{stv}).
}

\section{latent, id_col, classes_only}{

in some situations, it can be useful to identify the latent classes on
only one \code{type} of data while estimating the model parameters on both \code{types} of data. In such cases,
the input variable \code{latent} can be used to specify on which type of data the classification should be done.
If \verb{“cont”} or \verb{“dich”} is used, the input parameter \code{id_col} must be specified and gives the name,
i.e. a \verb{character string}, of the grouping variable for classification. Some groups may be removed from
the data, since they have only continuous or only dichotomous observations. Then in a first step,
a model is estimated only on the continuous/dichotomous data and the achieved classification is stored.
In a next step, model parameters are estimated separately for each identified class on both \code{types} of data
using this classification. The output object of \code{hyreg2} in this case is a \code{list} of k models.
Additionally, at position k+1 of the list, a data frame containing the corresponding classifications
from the first step is returned. Each element k in the list contains the estimated parameters for one
of the latent classes. When setting the input variable \code{classes_only} to \code{TRUE}, the second step is left
out and the estimated classes from step one are given as output.
}

\section{variables_both, variables_cont, variables_dich}{

It is possible to specify partial coefficients,
which are used only on continuous or dichotomous data.
\itemize{
\item Example:  Suppose different models should be specified for continuous and dichotomous  data:
\item Model continuous data: \code{y ~  x1 + x3}
\item Model dichotomous data: \code{y ~  x1 + x2}
\item The \code{formula} input to \code{hyreg2} must then include all parameters that occur in either model:
\code{y ~ x1 + x2 + x3}
\item The assignment of parameters to data types is then achieved via the input arguments \code{variables_both},
\code{variables_cont}, and \code{variables_dich}:
\item \code{variables_both} = \verb{“x1”},
\item \code{variables_cont} = \verb{“x3”} and
\item \code{variables_dich} = \verb{“x2”}.
\item Every variable included in the provided \code{formula} (except the grouping variable ) must appear in exactly
one of these vectors. One of the \code{variables_} vectors can also be \code{NULL}, if no variables should be used only on this type of the data.
}
}

\examples{

formula <- y ~  -1 + x1 + x2 + x3
formula_sigma <- y ~  x1 + x2 + x3

k <- 1
stv <- setNames(c(0.2,0,1,1),c(colnames(simulated_data_norm)[3:5],c("theta")))
stv_sigma <- setNames(c(0.2,0,1,1),c(colnames(simulated_data_norm)[3:5],c("(Intercept)")))
control = list(iter.max = 1000, verbose = 4)
rm(counter)
mod <- hyreg2_het(formula = formula,
                   formula_sigma = formula_sigma,
                    data =  simulated_data_norm,
                    type =  simulated_data_norm$type, # or "type"
                    stv = stv,
                    stv_sigma = stv_sigma,
                    k = k,
                    type_cont = "TTO",
                    type_dich = "DCE_A",
                    opt_method = "L-BFGS-B",
                    control = control,
                    latent = "both",
                    id_col = "id"
)
summary_hyreg2(mod)
}
\author{
Svenja Elkenkamp, Kim Rand and John Grosser
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{simulated_data_norm}
\alias{simulated_data_norm}
\title{simulated_data_norm}
\format{
\subsection{\code{simulated_data_norm}}{

A simulated data frame with 600 rows and 9 columns, following a combination of normal and binomial distribution
\describe{
\item{type}{type of data, "TTO" indicates normal distribution, "DCE_A" indicates binomial distribution}
\item{y}{result of the formula y ~ x1 + x2 + x3}
\item{x1, x2, x3}{random numbers from rnorm}
\item{class}{original class of the data point}
\item{id}{id number of observations to simulated different persons}
\item{y_non}{result of the formula y ~ (x1 \* beta1 + x2 \* beta3) \* (x1 \* beta1 + x3 \* beta3)}
\item{y_cens}{column y censored at 3}
...
}
}
}
\source{
{simulated with true parameter values:
Class 1: sigma  = 1.0, theta = 5 and c(x1,x2,x3) = c(0.5, -0.3, 0.8)
Class 2: sigma  = 0.5, theta = 2 and c(x1,x2,x3) = c(1.4, 2.3, -0.2)}
}
\usage{
simulated_data_norm
}
\description{
simulated_data_norm
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{simulated_data}
\alias{simulated_data}
\title{simulated_data}
\format{
\subsection{\code{simulated_data}}{

A simulated data frame with 480 rows and 25 columns, following a combination of normal and binomial distribution
\describe{
\item{type}{type of data, "TTO" indicates normal distribution, "DCE_A" indicates binomial distribution}
\item{y}{result of the formula y ~ -1 + mo2 + mo3 + ... + ad4 + ad5}
\item{mo2, mo3, mo4, mo5,
sc2, sc3, sc4, sc5,
ua2, ua3, ua4, ua5,
pd2, pd3, pd4, pd5,,
ad2, ad3, ad4, ad5,}{dummy variables for EQ5D data simulation}
\item{class}{original class of the data point}
\item{id}{id number of observations to simulated different persons}
\item{y_cens}{column y censored at 2 (upper boundary)}
...
}
}
}
\source{
{simulated with true parameter values:
Class 1: sigma  = 0.02, theta = 2 and
c(mo2,mo3,mo4,mo5) = c(0.001, 0.05, 0.08, 0.1),
c(sc2,sc3,sc4,sc5) = c(0.01, 0.2, 0.36, 0.5),
c(ua2,au3,ua4,ua5) = c(0.015, 0.25, 0.5, 0.8),
c(pd2,pd3,pd4,pd5) = c(0.1, 0.3, 0.4, 0.6),
c(ad2,ad3,ad4,ad5) = c(0.09, 0.19, 0.6, 0.7)

Class 2: sigma  = 0.1, theta = 3 and
c(mo2,mo3,mo4,mo5) = c(0.2, 0.4, 0.6, 0.8),
c(sc2,sc3,sc4,sc5) = c(0.1, 0.3, 0.4, 0.5),
c(ua2,au3,ua4,ua5) = c(0.2, 0.25, 0.6, 0.7),
c(pd2,pd3,pd4,pd5) = c(0.05, 0.2, 0.27, 0.8),
c(ad2,ad3,ad4,ad5) = c(0.15, 0.35, 0.4, 0.65)
}
}
\usage{
simulated_data
}
\description{
simulated_data
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/the_envirnoment.R
\docType{data}
\name{the}
\alias{the}
\title{creating environment for package internal objects}
\format{
An object of class \code{environment} of length 1.
}
\usage{
the
}
\description{
creating environment for package internal objects
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_hyreg2.R
\name{give_class}
\alias{give_class}
\title{decode classes by the model}
\usage{
give_class(data, model, id_col = NULL)
}
\arguments{
\item{data}{a \code{dataframe}, which was used to estimate the \code{model}}

\item{model}{a flexmix \code{model}object estimated using \code{\link[=hyreg2]{hyreg2()}} or \code{\link[=hyreg2_het]{hyreg2_het()}}}

\item{id_col}{\code{character}-string, name of grouping variable, which must be a column of the provided \code{data}.
the parameter must be specified, if the provided \code{model} was estimated under control for \code{groups}}
}
\value{
\code{dataframe} of two columns, first column named as provided \code{id_col}  or \code{"observation"} if \code{id_col} was not given as
an input. second column named \code{"mod_comp"} indicating the assigned class for this group or observation
}
\description{
This function can be used to decode the classified classes by the model generated using
\code{hyreg2} or \code{hyreg2_het}
}
\examples{
# estimate a model using simulated_data_norm

### using grouping variable id ####
formula <- y ~  -1 + x1 + x2 + x3 | id
k <- 1
stv <- setNames(c(0.2,0.2,0.2,1,1),c(colnames(simulated_data_norm)[3:5],c("sigma","theta")))
control <- list(iter.max = 1000, verbose = 4)

hyflex_mod <- hyreg2(formula = formula,
                    data =  simulated_data_norm,
                    type =  simulated_data_norm$type,
                    stv = stv,
                    k = k,
                    type_cont = "TTO",
                    type_dich = "DCE_A",
                    opt_method = "L-BFGS-B",
                    control = control,
                    latent = "both",
                    id_col = "id"
)
# use of function give_class
give_class(data = simulated_data_norm,
model = hyflex_mod,
id_col = "id")



}
\author{
Svenja Elkenkamp & John Grosser
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hyreg2.R
\name{summary_hyreg2}
\alias{summary_hyreg2}
\title{model summary for hyreg2 objects}
\usage{
summary_hyreg2(object)
}
\arguments{
\item{object}{\code{modelobject} generated with \code{\link[=hyreg2]{hyreg2()}} or \code{\link[=hyreg2_het]{hyreg2_het()}}}
}
\value{
\code{summary} object of \code{\link[bbmle:mle2]{bbmle::mle2()}} model,  Please note
that the outputs for \code{sigma} and \code{theta} are on a log-scale and have to be transformed using \code{exp()}to get the correct estimated values.
}
\description{
get model parameters of model generated by \code{hyreg2} or \code{hyreg2_het}
}
\examples{

formula <- y ~  -1 + x1 + x2 + x3 | id
k <- 1
stv <- setNames(c(0.2,0,1,1,1),c(colnames(simulated_data_norm)[3:5],c("sigma","theta")))
control = list(iter.max = 1000, verbose = 4)
rm(counter)
mod <- hyreg2(formula = formula,
                    data =  simulated_data_norm,
                    type =  simulated_data_norm$type,
                    stv = stv,
                    k = k,
                    type_cont = "TTO",
                    type_dich = "DCE_A",
                    opt_method = "L-BFGS-B",
                    control = control,
                    latent = "both",
                    id_col = "id"
)
summary_hyreg2(mod)

}
\author{
Svenja Elkenkamp
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hyreg2.R
\name{get_stv}
\alias{get_stv}
\title{extract parameter estimates as named vector}
\usage{
get_stv(mod, comp = "Comp.1")
}
\arguments{
\item{mod}{\code{model}output from \link{hyreg2} oder \link{hyreg2_het}. If \code{latent} was \code{"cont"} or \code{"dich"} only one element of the output list can be used.}

\item{comp}{\code{charachter}, default \code{"Comp.1"}. \code{"Comp.x"} indicating values from which model \code{component} (\code{x}) should be exported}
}
\value{
\verb{named vector} of parameter estimates from \code{mod}. Can be used as \code{stv} for additional model estimations using
\code{hyreg2} or \code{hyreg2_het}
}
\description{
function to export coefficient values and names from a \code{model} fitted by \code{hyreg2} or \code{hyreg2_het}
These values can be used as \code{stv} for a new model with \code{k > 1}
}
\examples{
formula <- y ~  -1 + x1 + x2 + x3 | id

k <- 1
stv <- setNames(c(0.2,0,1,1,1),c(colnames(simulated_data_norm)[3:5],c("sigma","theta")))
control = list(iter.max = 1000, verbose = 4)
rm(counter)
mod <- hyreg2(formula = formula,
                    data =  simulated_data_norm,
                    type =  simulated_data_norm$type,
                    stv = stv,
                    k = k,
                    type_cont = "TTO",
                    type_dich = "DCE_A",
                    opt_method = "L-BFGS-B",
                    control = control,
                    latent = "both",
                    id_col = "id"
)
new_stv <- get_stv(mod)

# these new_stv can be used in an other estimation using hyreg2 as stv
}
\author{
Svenja Elkenkamp
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_hyreg2.R
\name{plot_hyreg2}
\alias{plot_hyreg2}
\title{plot function for \code{hyreg2}}
\usage{
plot_hyreg2(
  data,
  x,
  y,
  id_col,
  class_df_model,
  type_to_plot = NULL,
  colors = NULL
)
}
\arguments{
\item{data}{a \code{dataframe}, which was used to estimate the \code{model} using \code{\link[=hyreg2]{hyreg2()}} or \code{\link[=hyreg2_het]{hyreg2_het()}}}

\item{x}{\code{charachter} string, column of \code{data} to be plotted on x-axis}

\item{y}{\code{charachter} string, column of \code{data} to be plotted on y-axis}

\item{id_col}{\code{charachter} sting, grouping variable, same as was given in \code{model}.
if model was estimated without grouping, see Details}

\item{class_df_model}{\code{dataframe} of two columns indicating which group belongs to which class,
first column named as input \code{id_col}, second column named \code{"mod_comp"}.
this input can be generated using the \code{\link[=give_class]{give_class()}} function, see Details.}

\item{type_to_plot}{\code{list} of two \code{charachter} elements. First: \code{columnname} of column containing indicator for \code{type} of \code{data},
Second: \code{value} of column \code{type}, that should be used for the plot, see details of \code{\link[=hyreg2]{hyreg2()}} inputs \code{type} and \code{type_cont},\code{type_dich}}

\item{colors}{\code{charachter} vector, colors to be used in \code{ggplot}, default \code{NULL} - than colors are choosen automatically}
}
\value{
\code{ggplot} object visualizing x against y by classes
}
\description{
Function to visualize model results by \code{hyreg2} or \code{hygre2_het}
}
\details{
\code{id_col} must be provided anyway, even if the model was estimated without grouping variable.
Since there might be no grouping varibale in the \code{data}, we recommend to create a new column called \code{"observation"}
in data using the \code{rownames}/\code{observationnumbers} as \code{charachter} values and use this column as
input for \code{id_col} in \code{plot_hyreg2}, additionally you can use \code{class_df_model} =  \code{give_class(data,model,"observation")},
see example
}
\examples{
# estimate a model using simulated_data_rnorm

formula <- y ~  -1 + x1 + x2 + x3 | id
k <- 2
stv <- setNames(c(0.2,0.2,0.2,1,1),c(colnames(simulated_data_norm)[3:5],c("sigma","theta")))
control <- list(iter.max = 1000, verbose = 4)

hyflex_mod <- hyreg2(formula = formula,
                    data =  simulated_data_norm,
                    type =  simulated_data_norm$type,
                    stv = stv,
                    k = k,
                    type_cont = "TTO",
                    type_dich = "DCE_A",
                    opt_method = "L-BFGS-B",
                    control = control,
                    latent = "cont",
                    id_col = "id"
)
# plotting the variables id against y
plot_hyreg2(data = simulated_data_norm,
          x = "id",
          y = "y",
          id_col = "id",
          class_df_model = give_class(data = simulated_data_norm,
                                model = hyflex_mod,
                                id = "id"))


}
\author{
Svenja Elkenkamp & John Grosser
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/FLXMRhyreg_het.R
\name{FLXMRhyreg_het}
\alias{FLXMRhyreg_het}
\title{M-step driver to be used in flexmix accounting for heteroscedastisity}
\usage{
FLXMRhyreg_het(
  data,
  formula = . ~ .,
  formula_sigma = formula_sigma,
  family = c("hyreg"),
  type = NULL,
  type_cont = NULL,
  type_dich = NULL,
  variables_both = NULL,
  variables_cont = NULL,
  variables_dich = NULL,
  stv = NULL,
  stv_sigma = NULL,
  offset = NULL,
  opt_method = "BFGS",
  optimizer = "optim",
  lower = -Inf,
  upper = Inf,
  ...
)
}
\arguments{
\item{data}{a \code{data.frame} containing the data, see Details of \link{hyreg2_het}}

\item{formula}{linear model \code{formula}}

\item{formula_sigma}{\code{formula} for estimation of \code{sigma} to account for heteroscedasticity, see Details \link{hyreg2_het}}

\item{family}{default \code{"hyreg"}, needed for \code{\link[flexmix:flexmix]{flexmix::flexmix()}}}

\item{type}{\code{character} vector containing the indicator whether that datapoint (row)
contains continuous or dichotomous data, see Details of \link{hyreg2_het}}

\item{type_cont}{value of \code{type} referring to continuous \code{data}, see Details of \link{hyreg2_het}}

\item{type_dich}{value of \code{type} referring to dichotomous \code{data}, see Details of \link{hyreg2_het}}

\item{variables_both}{\code{character} vector; variables to be fitted on both continuous and dichotomous data.
see Details of \link{hyreg2_het}}

\item{variables_cont}{\code{character} vector; variables to be fitted only on continous data. see Details of \link{hyreg2_het}}

\item{variables_dich}{character vactor; variables to be fitted only on dichotomous data. see Details of \link{hyreg2_het}}

\item{stv}{\verb{named vector} or \code{list} of named vectors containing start values for all coefficients from
\code{formula}, including \code{theta}, see Details of \link{hyreg2_het}}

\item{stv_sigma}{\verb{named vector} with start values for sigma estimation.
Have to be the same variables as given in \code{formula_sigma}, see Details of \link{hyreg2_het}}

\item{offset}{offset as in \code{\link[flexmix:flexmix]{flexmix::flexmix()}}, default \code{NULL}}

\item{opt_method}{\code{character}, optimization method to be used in \code{optimizer}, default \code{"BFGS"}}

\item{optimizer}{\code{character}, optimizer to be used in \code{\link[bbmle:mle2]{bbmle::mle2()}}, default \code{"optim"}}

\item{lower}{lower bound for censored data. If this is used, \code{opt_method} must be
set to \code{"L-BFGS-B"}, default \code{-INF},}

\item{upper}{upper bound for censored data. If this is used, \code{opt_method} must be
set to \code{"L-BFGS-B"},default \code{INF}}

\item{...}{additional arguments for \code{\link[flexmix:flexmix]{flexmix::flexmix()}} or \code{\link[bbmle:mle2]{bbmle::mle2()}}}
}
\value{
a \code{model} object, that can be used in \link{hyreg2_het} as input for parameter \code{model} in \code{\link[flexmix:flexmix]{flexmix::flexmix()}}
}
\description{
Function used in flexmix M-Step to estimate hybrid model accounting for heteroscedastisity
}
\examples{

formula <- y ~  -1 + x1 + x2 + x3
formula_sigma <- y ~ x1 + x2 + x3
stv <- setNames(c(0.2,0,1,1),c(colnames(simulated_data_norm)[3:5],c("theta")))
stv_sigma <- setNames(c(0.2,0.2,0.1,1),c(colnames(simulated_data_norm)[3:5],c("(Intercept)")))


x <- model.matrix(formula,simulated_data_norm)
y <- simulated_data_norm$y
w <- 1
model <- FLXMRhyreg_het( data = simulated_data_norm,
                     formula = formula,
                     formula_sigma = formula_sigma,
                    family=c("hyreg"),
                    type =  simulated_data_norm$type,
                    stv = stv,
                    stv_sigma = stv_sigma,
                    type_cont = "TTO",
                    type_dich = "DCE_A",
                    opt_method = "L-BFGS-B",
                    control = list(iter.max = 1000, verbose = 4),
                    offset = NULL,
                    optimizer = "optim",
                    variables_both =  names(stv)[!is.element(names(stv),c("theta"))],
                    variables_cont = NULL,
                    variables_dich = NULL,
                    lower = -Inf,
                    upper = Inf,
)
}
\author{
Svenja Elkenkamp and Kim Rand
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/FLXMRhyreg.R
\name{FLXMRhyreg}
\alias{FLXMRhyreg}
\title{M-step driver to be used in flexmix}
\usage{
FLXMRhyreg(
  formula = . ~ .,
  family = c("hyreg"),
  type = NULL,
  type_cont = NULL,
  type_dich = NULL,
  variables_both = NULL,
  variables_cont = NULL,
  variables_dich = NULL,
  stv = NULL,
  offset = NULL,
  opt_method = "BFGS",
  optimizer = "optim",
  lower = -Inf,
  upper = Inf,
  formula_type_classic = TRUE,
  ...
)
}
\arguments{
\item{formula}{model \code{formula}, automatically provided by \code{hyreg2} and \code{\link[flexmix:flexmix]{flexmix::flexmix()}}}

\item{family}{default \code{"hyreg"}, needed for \code{\link[flexmix:flexmix]{flexmix::flexmix()}}}

\item{type}{\code{character} vector containing the indicator whether that datapoint (row)
contains continuous or dichotomous data, see Details of \link{hyreg2}}

\item{type_cont}{value of \code{type} referring to continuous \code{data}, see Details of \link{hyreg2}}

\item{type_dich}{value of \code{type} referring to dichotomous \code{data}, see Details of \link{hyreg2}}

\item{variables_both}{\code{character} vector; variables to be fitted on both continuous and dichotomous data.
see Details of \link{hyreg2}}

\item{variables_cont}{\code{character} vector; variables to be fitted only on continous data. see Details of \link{hyreg2}}

\item{variables_dich}{character vactor; variables to be fitted only on dichotomous data. see Details of \link{hyreg2}}

\item{stv}{\verb{named vector} or \code{list} of named vectors containing start values for all coefficients from
\code{formula}, including \code{theta}, see Details of \link{hyreg2}}

\item{offset}{offset as in \code{\link[flexmix:flexmix]{flexmix::flexmix()}}, default \code{NULL}}

\item{opt_method}{\code{character}, optimization method to be used in \code{optimizer}, default \code{"BFGS"}}

\item{optimizer}{\code{character}, optimizer to be used in \code{\link[bbmle:mle2]{bbmle::mle2()}}, default \code{"optim"}}

\item{lower}{lower bound for censored data. If this is used, \code{opt_method} must be
set to \code{"L-BFGS-B"}, default \code{-INF},}

\item{upper}{upper bound for censored data. If this is used, \code{opt_method} must be
set to \code{"L-BFGS-B"},default \code{INF}}

\item{formula_type_classic}{\code{logical}; is the provided \code{formula} a typical R formula containing only variables
or does it include variables and parameters? default \code{TRUE}}

\item{...}{additional arguments for \code{\link[flexmix:flexmix]{flexmix::flexmix()}} or \code{\link[bbmle:mle2]{bbmle::mle2()}}}
}
\value{
a \code{model} object, that can be used in \link{hyreg2} as input for parameter \code{model} in \code{\link[flexmix:flexmix]{flexmix::flexmix()}}

a model object, that can be used in hyreg2 as input for parameter model in flexmix::flexmix
}
\description{
Function used in flexmix M-Step to estimate hybrid model
}
\examples{

formula <- y ~  -1 + x1 + x2 + x3
the$k <- 2
stv <- setNames(c(0.2,0,1,1,1),c(colnames(simulated_data_norm)[3:5],c("sigma","theta")))



x <- model.matrix(formula,simulated_data_norm)
y <- simulated_data_norm$y
w <- 1
model <- FLXMRhyreg(formula = formula,
                    family=c("hyreg"),
                    type =  simulated_data_norm$type,
                    stv = stv,
                    type_cont = "TTO",
                    type_dich = "DCE_A",
                    opt_method = "L-BFGS-B",
                    control = list(iter.max = 1000, verbose = 4),
                    offset = NULL,
                    optimizer = "optim",
                    variables_both =  names(stv)[!is.element(names(stv),c("sigma","theta"))],
                    variables_cont = NULL,
                    variables_dich = NULL,
                    lower = -Inf,
                    upper = Inf,
)
}
\author{
Svenja Elkenkamp and Kim Rand
}
